/*------------------------------------------------------------------------------
 bin/djv_view/app.h

 Copyright (c) 2004-2008 Darby Johnston. All rights reserved.
 Use is subject to license terms.
------------------------------------------------------------------------------*/

#ifndef DJV_VIEW_APP
#define DJV_VIEW_APP

#include "file.h"
#include "image.h"
#include "image_cache.h"
#include "image_display.h"
#include "image_histogram.h"
#include "image_info.h"
#include "image_magnify.h"
#include "image_sample.h"
#include "playback.h"
#include "shortcut.h"
#include "window.h"
#include "view.h"
#include <djv_ui/app.h>

namespace djv_view {
using namespace djv_base;

//------------------------------------------------------------------------------
// Application

class App : public djv_ui::app::App
{
public:

  App(int & argc, char ** argv);
  ~App();
  
  // Command line.

  void cmdln(Str &) throw (Error);
  Str cmdln_help() const;
  
  // Global.
  
  shortcut::Prefs * shortcut_prefs() const;
  image_cache::Cache * file_cache() const;
  file::Save * file_save() const;
  file::Prefs * file_prefs() const;
  window::Factory * window_factory() const;
  window::Prefs * window_prefs() const;
  view::Prefs * view_prefs() const;
  image::Prefs * image_prefs() const;
  image_display::Dialog * image_display() const;
  image_histogram::Dialog * image_histogram() const;
  image_info::Dialog * image_info() const;
  image_magnify::Dialog * image_magnify() const;
  image_sample::Dialog * image_sample() const;
  playback::Prefs * playback_prefs() const;
  playback::Global * playback_global() const;

private:
  
  window::Widget * window() const;
  
  struct Cmdln
  {
    std::auto_ptr<int> file_layer;
    std::auto_ptr<djv_image::data::PROXY> file_proxy;
    std::auto_ptr<bool> file_cache;
    std::auto_ptr<bool> window_fullscreen;
    std::auto_ptr<bool> window_toolbar, window_playbar;
    std::auto_ptr<view::GRID> view_grid;
    std::auto_ptr<bool> view_hud;
    std::auto_ptr<playback::PLAYBACK> playback;
    std::auto_ptr<int> playback_frame;
    std::auto_ptr<djv_image::time::Speed> playback_speed;
  } _cmdln;

  List<Str> _input;

  std::auto_ptr<shortcut::Prefs> _shortcut_prefs;
  std::auto_ptr<image_cache::Cache> _file_cache;
  std::auto_ptr<file::Save> _file_save;
  std::auto_ptr<file::Prefs> _file_prefs;
  std::auto_ptr<window::Prefs> _window_prefs;
  std::auto_ptr<view::Prefs> _view_prefs;
  std::auto_ptr<image::Prefs> _image_prefs;
  djv_ui::widget::Auto_Ptr<image_display::Dialog> _image_display;
  djv_ui::widget::Auto_Ptr<image_histogram::Dialog> _image_histogram;
  djv_ui::widget::Auto_Ptr<image_info::Dialog> _image_info;
  djv_ui::widget::Auto_Ptr<image_magnify::Dialog> _image_magnify;
  djv_ui::widget::Auto_Ptr<image_sample::Dialog> _image_sample;
  std::auto_ptr<playback::Prefs> _playback_prefs;
  std::auto_ptr<playback::Global> _playback_global;
  std::auto_ptr<window::Factory> _window_factory;
};

//------------------------------------------------------------------------------
// Labels

extern const Str
  label_cmdln_help;

//------------------------------------------------------------------------------
// Global

inline App * app() { return static_cast<App *>(djv_ui::app::global()); }

}

#endif

