/*------------------------------------------------------------------------------
 bin/djv_view/image_prefs.cxx

 Copyright (c) 2004-2008 Darby Johnston. All rights reserved.
 Use is subject to license terms.
------------------------------------------------------------------------------*/

#include "app.h"
#include "image.h"
#include <djv_ui/prefs.h>

namespace djv_view {
namespace image {

Prefs::Prefs() :
  mirror_sig(this),
  scale_sig(this),
  rotate_sig(this),
  profile_sig(this),
  channel_sig(this),
  display_sig(this),
  display_value_sig(this),
  display_list_sig(this),
  _frame_store_file_reload(false),
  _scale(SCALE(0)),
  _rotate(ROTATE(0)),
  _profile(true),
  _channel(CHANNEL(0)),
  _display(-1)
{
  djv_ui::prefs::Prefs prefs(djv_ui::prefs::prefs(), "image");

  djv_ui::prefs::get(&prefs, "frame_store_file_reload",
    &_frame_store_file_reload);
  djv_ui::prefs::get(&prefs, "mirror", &_mirror);
  djv_ui::prefs::get(&prefs, "scale", &_scale);
  djv_ui::prefs::get(&prefs, "rotate", &_rotate);
  djv_ui::prefs::get(&prefs, "profile", &_profile);
  djv_ui::prefs::get(&prefs, "channel", &_channel);

  djv_ui::prefs::Prefs display_prefs(prefs, "display");
  djv_ui::prefs::get(&display_prefs, "value", &_display);
  int display_list_size = 0;
  djv_ui::prefs::get(&display_prefs, "size", &display_list_size);
  for (int i = 0; i < display_list_size; ++i)
  {
    Display value;
    djv_ui::prefs::get(&display_prefs, str::Format("%%").arg(i), &value);
    const djv_image::Image * image = lut_load(value.lut_file);
    value.lut = image ? *image : djv_image::Image();
    _display_list += value;
  }
}
  
Prefs::~Prefs()
{
  djv_ui::prefs::Prefs prefs(djv_ui::prefs::prefs(), "image");

  djv_ui::prefs::set(&prefs, "frame_store_file_reload",
    _frame_store_file_reload);
  djv_ui::prefs::set(&prefs, "mirror", _mirror);
  djv_ui::prefs::set(&prefs, "scale", _scale);
  djv_ui::prefs::set(&prefs, "rotate", _rotate);
  djv_ui::prefs::set(&prefs, "profile", _profile);
  djv_ui::prefs::set(&prefs, "channel", _channel);

  djv_ui::prefs::Prefs display_prefs(prefs, "display");
  djv_ui::prefs::set(&display_prefs, "value",_display);
  djv_ui::prefs::set(&display_prefs, "size", int(_display_list.size()));
  for (uint i = 0; i < _display_list.size(); ++i)
    djv_ui::prefs::set(&display_prefs, str::Format("%%").arg(i),
      _display_list[i]);
}

void Prefs::frame_store_file_reload(bool in)
{
  _frame_store_file_reload = in;
}

bool Prefs::frame_store_file_reload() const { return _frame_store_file_reload; }

void Prefs::mirror(const Vec2b & in)
{
  if (in == _mirror) return;
  
  _mirror = in;
  
  mirror_sig.emit(_mirror);
}

const Vec2b & Prefs::mirror() const { return _mirror; }

void Prefs::scale(SCALE in)
{
  if (in == _scale) return;
  
  _scale = in;
  
  scale_sig.emit(_scale);
}

image::SCALE Prefs::scale() const { return _scale; }

void Prefs::rotate(ROTATE in)
{
  if (in == _rotate) return;
  
  _rotate = in;
  
  rotate_sig.emit(_rotate);
}

image::ROTATE Prefs::rotate() const { return _rotate; }

void Prefs::profile(bool in)
{
  if (in == _profile) return;
  
  _profile = in;
  
  profile_sig.emit(_profile);
}

bool Prefs::profile() const { return _profile; }

void Prefs::channel(CHANNEL in)
{
  if (in == _channel) return;
  
  _channel = in;
  
  channel_sig.emit(_channel);
}

image::CHANNEL Prefs::channel() const { return _channel; }

void Prefs::display(int in)
{
  if (in == _display) return;
  
  _display = in;
  
  display_sig.emit(_display);
  display_value_sig.emit(
    (_display != -1) ? _display_list[_display] : Display());
}

int Prefs::display() const { return _display; }

image::Display Prefs::display_value() const
{
  return _display != -1 ? _display_list[_display] : image::Display();
}

void Prefs::display_list(const List<Display> & in)
{
  _display_list = in;
  _display = math::min(_display, int(_display_list.size()) - 1);
  
  display_list_sig.emit(true);
}

const List<image::Display> & Prefs::display_list() const
{
  return _display_list;
}

List<Str> Prefs::display_name_list() const
{
  List<Str> out;
  for (uint i = 0; i < _display_list.size(); ++i)
    out += _display_list[i].name;
  return out;
}

}
}

