/*------------------------------------------------------------------------------
 bin/djv_view/view_prefs.cxx

 Copyright (c) 2004-2008 Darby Johnston. All rights reserved.
 Use is subject to license terms.
------------------------------------------------------------------------------*/

#undef DJV_DBG
//#define DJV_DBG

#include "app.h"
#include "view.h"
#include <djv_ui/prefs.h>

namespace djv_view {
namespace view {

Prefs::Prefs() :
  resize_sig(this),
  grid_sig(this),
  hud_sig(this),
  _background(djv_image::Color(0.0)),
  _resize(RESIZE(0)),
  _grid(GRID(0)),
  _grid_color(djv_image::Color(0.25)),
  _hud(false),
  _hud_option(true, _HUD_OPTION_SIZE),
  _hud_background(HUD_BACKGROUND_SHADOW),
  _hud_color_foreground(djv_image::Color(1.0)),
  _hud_color_background(djv_image::Color(0.0))
{
  DJV_DBG_OBJ("Prefs::Prefs");
  
  djv_ui::prefs::Prefs prefs(djv_ui::prefs::prefs(), "view");
  djv_ui::prefs::get(&prefs, "background", &_background);
  djv_ui::prefs::get(&prefs, "resize", &_resize);
  djv_ui::prefs::get(&prefs, "grid", &_grid);
  djv_ui::prefs::get(&prefs, "grid_color", &_grid_color);
  djv_ui::prefs::get(&prefs, "hud", &_hud);
  djv_ui::prefs::get(&prefs, "hud_option", &_hud_option);
  djv_ui::prefs::get(&prefs, "hud_background", &_hud_background);
  djv_ui::prefs::get(&prefs, "hud_color_foreground", &_hud_color_foreground);
  djv_ui::prefs::get(&prefs, "hud_color_background", &_hud_color_background);
}

Prefs::~Prefs()
{
  DJV_DBG_OBJ("Prefs::~Prefs");

  djv_ui::prefs::Prefs prefs(djv_ui::prefs::prefs(), "view");
  djv_ui::prefs::set(&prefs, "background", _background);
  djv_ui::prefs::set(&prefs, "resize", _resize);
  djv_ui::prefs::set(&prefs, "grid", _grid);
  djv_ui::prefs::set(&prefs, "grid_color", _grid_color);
  djv_ui::prefs::set(&prefs, "hud", _hud);
  djv_ui::prefs::set(&prefs, "hud_option", _hud_option);
  djv_ui::prefs::set(&prefs, "hud_background", _hud_background);
  djv_ui::prefs::set(&prefs, "hud_color_foreground", _hud_color_foreground);
  djv_ui::prefs::set(&prefs, "hud_color_background", _hud_color_background);
}

void Prefs::background(const djv_image::Color & in)
{
  if (in == _background) return;
  
  _background = in;
  
  app()->window_update();
}

const djv_image::Color & Prefs::background() const { return _background; }

void Prefs::resize(RESIZE in)
{
  if (in == _resize) return;
  
  _resize = in;
  
  resize_sig.emit(_resize);
}

RESIZE Prefs::resize() const { return _resize; }

void Prefs::grid(GRID in)
{
  if (in == _grid) return;
  
  _grid = in;
  
  grid_sig.emit(_grid);
}

GRID Prefs::grid() const { return _grid; }

void Prefs::grid_color(const djv_image::Color & in)
{
  if (in == _grid_color) return;
  
  _grid_color = in;
  
  app()->window_update();
}

const djv_image::Color & Prefs::grid_color() const { return _grid_color; }

void Prefs::hud(bool in)
{
  if (in == _hud) return;
  
  _hud = in;
  
  hud_sig.emit(_hud);
}

bool Prefs::hud() const { return _hud; }

void Prefs::hud_option(HUD_OPTION option, bool in)
{
  if (in == _hud_option[option]) return;
  
  _hud_option[option] = in;
  
  app()->window_update();
}

List<bool> Prefs::hud_option() const { return _hud_option; }

bool Prefs::hud_option(HUD_OPTION in) const { return _hud_option[in]; }

void Prefs::hud_background(HUD_BACKGROUND in)
{
  if (in == _hud_background) return;
  
  _hud_background = in;
  
  app()->window_update();
}

HUD_BACKGROUND Prefs::hud_background() const { return _hud_background; }

void Prefs::hud_color_foreground(const djv_image::Color & in)
{
  if (in == _hud_color_foreground) return;
  
  _hud_color_foreground = in;
  
  app()->window_update();
}

const djv_image::Color & Prefs::hud_color_foreground() const
  { return _hud_color_foreground; }

void Prefs::hud_color_background(const djv_image::Color & in)
{
  if (in == _hud_color_background) return;
  
  _hud_color_background = in;
  
  app()->window_update();
}

const djv_image::Color & Prefs::hud_color_background() const
  { return _hud_color_background; }

}
}

