/*------------------------------------------------------------------------------
 bin/djv_view/window_prefs_widget.cxx

 Copyright (c) 2004-2008 Darby Johnston. All rights reserved.
 Use is subject to license terms.
------------------------------------------------------------------------------*/

#undef DJV_DBG
//#define DJV_DBG

#include "window.h"
#include <djv_ui/button.h>
#include <djv_ui/choice.h>
#include <djv_ui/group.h>
#include <djv_ui/label.h>
#include <djv_ui/layout_row.h>
#include <djv_ui/menu.h>
#include <djv_ui/style.h>
#include <djv_ui/text.h>

namespace djv_view {
namespace window {

Prefs_Widget::Prefs_Widget()
{
  using namespace djv_ui;
  
  // Widgets.
  
  group::Box * resize_group =
    new group::Box(prefs_resize_group);
  button::Check * resize_fit_widget =
    new button::Check(prefs_resize_fit);
  choice::Widget * resize_max_widget =
    new choice::Widget(label_resize_max(), prefs_resize_max);
  
  group::Box * controls_group =
    new group::Box(prefs_controls_group);
  button::Check * toolbar_widget =
    new button::Check(prefs_toolbar);
  button::Check * playbar_widget =
    new button::Check(prefs_playbar);

  // Layout.
  
  Layout_V * layout, * layout_group;
  
  layout = new Layout_V(this);
  
  layout->add(resize_group);
  layout_group = new Layout_V(resize_group);
  layout_group->spacing(style::global()->spacing_large());
  layout_group->add(resize_fit_widget);
  layout_group->add(resize_max_widget);
  
  layout->add(controls_group);
  layout_group = new Layout_V(controls_group);
  layout_group->spacing(style::global()->spacing_large());
  layout_group->add(toolbar_widget);
  layout_group->add(playbar_widget);

  layout->add_stretch();
  
  // Initialize.
  
  resize_fit_widget->set(prefs()->resize_fit());
  resize_max_widget->set(prefs()->resize_max());
  toolbar_widget->set(prefs()->toolbar());
  playbar_widget->set(prefs()->playbar());

  // Callbacks.

  resize_fit_widget->sig.set(this, resize_fit_callback);
  resize_max_widget->sig.set(this, resize_max_callback);
  toolbar_widget->sig.set(this, toolbar_callback);
  playbar_widget->sig.set(this, playbar_callback);
}

void Prefs_Widget::resize_fit_callback(bool in)
  { prefs()->resize_fit(in); }

void Prefs_Widget::resize_max_callback(int in)
  { prefs()->resize_max(RESIZE_MAX(in)); }

void Prefs_Widget::toolbar_callback(bool in)
  { prefs()->toolbar(in); }

void Prefs_Widget::playbar_callback(bool in)
  { prefs()->playbar(in); }

}
}

