/*------------------------------------------------------------------------------
 include/djv_base/app.h
 
 Command Line Application
 ------------------------
 
 Copyright (c) 2004-2008 Darby Johnston. All rights reserved.
 Use is subject to license terms.
------------------------------------------------------------------------------*/

#ifndef DJV_APP_H
#define DJV_APP_H

#include <djv_base/base.h>

namespace djv_base {
namespace app {

//------------------------------------------------------------------------------
// Options

// Exit value.

enum EXIT
{
  EXIT_DEFAULT,
  EXIT_ERROR,
  EXIT_HELP,
  EXIT_INFO,
  EXIT_ABOUT
};

//------------------------------------------------------------------------------
// Application

class App
{
public:

  App(const Str & name, int & argc, char ** argv);

  virtual ~App();

  // Execution.

  virtual int exec();
  
  // Exit value.
  
  virtual void exit(EXIT);
  EXIT exit() const;

  // Information.
  
  const Str & name() const;
  const Str & path() const;
  virtual Str info() const;
  virtual Str about() const;
  
  // Messages.

  virtual void message(const Str &) const;
  virtual void error(const Error &, const Str & message = Str()) const;
  virtual void print(const Str &, bool newline = true) const;
  virtual void separator() const;

  // Command line.

  void cmdln(Str &) throw (Error);
  const Str & cmdln_name() const;
  const Str & cmdln_arg() const;
  virtual Str cmdln_help() const;

private:

  EXIT _exit;
  Str _name;
  Str _path;
  Str _cmdln_name, _cmdln_arg;
  bool _endline, _separator;
};

//------------------------------------------------------------------------------
// Labels

extern const Str
  label_info,
  label_about,
  label_cmdln_help;

extern const Str
  error_cmdln,
  error_format;

//------------------------------------------------------------------------------
// Global

App * global();

}
}

#endif

