/*------------------------------------------------------------------------------
 include/djv_base/box2_.h

 Copyright (c) 2004-2008 Darby Johnston. All rights reserved.
 Use is subject to license terms.
------------------------------------------------------------------------------*/

namespace djv_base {
namespace box {

template<typename T>
const int Box<T, 2>::dimension = 2;

#define _BOX2_INIT \
  \
  x(this->position.e[0]), \
  y(this->position.e[1]), \
  w(this->size.e[0]), \
  h(this->size.e[1])

template<typename T>
inline Box<T, 2>::Box() :
  _BOX2_INIT
{}

template<typename T>
inline Box<T, 2>::Box(const Box<T, 2> & in) :
  position(in.position),
  size(in.size),
  _BOX2_INIT
{}

template<typename T>
inline Box<T, 2>::Box(const Vec<T, 2> & position, const Vec<T, 2> & size) :
  position(position),
  size(size),
  _BOX2_INIT
{}

template<typename T>
inline Box<T, 2>::Box(const Vec<T, 2> & size) :
  size(size),
  _BOX2_INIT
{}

template<typename T>
inline Box<T, 2>::Box(T _x, T _y, T _w, T _h) :
  _BOX2_INIT
{
  x = _x;
  y = _y;
  w = _w;
  h = _h;
}

template<typename T>
inline Box<T, 2>::Box(T _w, T _h) :
  _BOX2_INIT
{
  w = _w;
  h = _h;
}

template<typename T>
inline Box<T, 2> & Box<T, 2>::operator = (const Box<T, 2> & in)
{
  if (&in != this) set(in);
  return *this;
}

template<typename T>
inline void Box<T, 2>::set(const Box<T, 2> & in)
{
  position = in.position;
  size = in.size;
}

template<typename T>
inline void Box<T, 2>::position1(const Vec<T, 2> & in)
{
  size = in - position;
}

template<typename T>
inline Vec<T, 2> Box<T, 2>::position1() const
{
  return position + size;
}

template<typename T>
inline void Box<T, 2>::zero()
{
  position.zero();
  size.zero();
}

// XXX Specialization for integer types.

template<>
inline void Box<int, 2>::position1(const Vec<int, 2> & in)
{
  size = in - position + 1;
}

template<>
inline Vec<int, 2> Box<int, 2>::position1() const
{
  return position + size - 1;
}

template<typename T>
inline Box<T, 2>::operator Box<int, 2> () const
{
  return Box<int, 2>(int(x), int(y), int(w), int(h));
}

template<typename T>
inline Box<T, 2>::operator Box<double, 2> () const
{
  return Box<double, 2>(double(x), double(y), double(w), double(h));
}

}
}
