/*------------------------------------------------------------------------------
 include/djv_base/mtx3_.h

 Copyright (c) 2004-2008 Darby Johnston. All rights reserved.
 Use is subject to license terms.
------------------------------------------------------------------------------*/

namespace djv_base {
namespace mtx {

template<typename T>
const int Mtx<T, 3>::dimension = 3;
template<typename T>
const int Mtx<T, 3>::dimension_x2 = 3 * 3;

template<typename T>
inline Mtx<T, 3>::Mtx()
{
  identity();
}

template<typename T>
inline Mtx<T, 3>::Mtx(const Mtx<T, 3> & in)
{
  set(in);
}

template<typename T>
inline Mtx<T, 3>::Mtx(
  T _0, T _1, T _2,
  T _3, T _4, T _5,
  T _6, T _7, T _8
) {
  set(
    _0, _1, _2,
    _3, _4, _5,
    _6, _7, _8);
}

template<typename T>
inline Mtx<T, 3>::Mtx(const T * in)
{
  set(in);
}

template<typename T>
inline Mtx<T, 3> & Mtx<T, 3>::operator = (const Mtx<T, 3> & in)
{
  if (&in != this) set(in);
  return *this;
}

template<typename T>
inline void Mtx<T, 3>::set(const Mtx<T, 3> & in)
{
  for (int i = 0; i < dimension_x2; ++i)
    e[i] = in.e[i];
}

template<typename T>
inline void Mtx<T, 3>::set(
  T _0, T _1, T _2,
  T _3, T _4, T _5,
  T _6, T _7, T _8
) {
  T * p = Mtx<T, 3>::e;
  p[0] = _0; p[1] = _1; p[2] = _2;
  p[3] = _3; p[4] = _4; p[5] = _5;
  p[6] = _6; p[7] = _7; p[8] = _8;
}

template<typename T>
inline void Mtx<T, 3>::set(const T * in)
{
  for (int i = 0; i < dimension_x2; ++i)
    e[i] = in[i];
}

template<typename T>
inline void Mtx<T, 3>::identity()
{
  for (int i = 0; i < 3; ++i)
    for (int j = 0; j < 3; ++j)
      e[i * 3 + j] = (i == j) ? 1.0 : 0.0;
}

template<typename T>
inline void Mtx<T, 3>::zero()
{
  for (int i = 0; i < dimension_x2; ++i)
    e[i] = 0.0;
}

}
}

