/*------------------------------------------------------------------------------
 include/djv_base/vec3_.h

 Copyright (c) 2004-2008 Darby Johnston. All rights reserved.
 Use is subject to license terms.
------------------------------------------------------------------------------*/

namespace djv_base {
namespace vec {

template<typename T>
const int Vec<T, 3>::dimension = 3;

#define _VEC3_INIT \
  \
  x(e[0]), \
  y(e[1]), \
  z(e[2])

template<typename T>
inline Vec<T, 3>::Vec() :
  _VEC3_INIT
{
  x = y = 0;
}

template<typename T>
inline Vec<T, 3>::Vec(const Vec<T, 3> & in) :
  _VEC3_INIT
{
  x = in.x;
  y = in.y;
  z = in.z;
}

template<typename T>
inline Vec<T, 3>::Vec(T _x, T _y, T _z) :
  _VEC3_INIT
{
  x = _x;
  y = _y;
  z = _z;
}

template<typename T>
inline Vec<T, 3>::Vec(T in) :
  _VEC3_INIT
{
  x = y = z = in;
}
  
template<typename T>
inline Vec<T, 3> & Vec<T, 3>::operator = (const Vec<T, 3> & in)
{
  if (&in != this) set(in);
  return *this;
}

template<typename T>
inline void Vec<T, 3>::set(const Vec<T, 3> & in)
{
  x = in.x;
  y = in.y;
  z = in.z;
}

template<typename T>
inline void Vec<T, 3>::set(T in)
{
  x = y = z = in;
}

template<typename T>
inline void Vec<T, 3>::zero()
{
  x = y = z = T(0);
}

#define _VEC3_OP(IN) \
  \
  template<typename T> \
  inline Vec<T, 3> & Vec<T, 3>::operator IN (const Vec<T, 3> & in) \
  { \
    x IN in.x; \
    y IN in.y; \
    z IN in.z; \
    return *this; \
  }

_VEC3_OP(+=)
_VEC3_OP(-=)
_VEC3_OP(*=)
_VEC3_OP(/=)

#define _VEC3_OP2(IN) \
  \
  template<typename T> \
  inline Vec<T, 3> & Vec<T, 3>::operator IN (T in) \
  { \
    x IN in; \
    y IN in; \
    z IN in; \
    return *this; \
  }

_VEC3_OP2(+=)
_VEC3_OP2(-=)
_VEC3_OP2(*=)
_VEC3_OP2(/=)

template<typename T>
inline Vec<T, 3>::operator Vec<int, 3> () const
{
  return Vec<int, 3>(int(x), int(y), int(z));
}

template<typename T>
inline Vec<T, 3>::operator Vec<double, 3> () const
{
  return Vec<double, 3>(double(x), double(y), double(z));
}

}
}

