/*------------------------------------------------------------------------------
 include/djv_image/gl_offscreen.h
 
 OpenGL Offscreen Rendering
 --------------------------
 
 Copyright (c) 2004-2008 Darby Johnston. All rights reserved.
 Use is subject to license terms.
------------------------------------------------------------------------------*/

#ifndef DJV_IMAGE_GL_OFFSCREEN_H
#define DJV_IMAGE_GL_OFFSCREEN_H

#include <djv_image/gl.h>
#include <djv_image/data.h>

namespace djv_image {
namespace gl_offscreen {

//------------------------------------------------------------------------------
// Options

enum BUFFER
{
  PBUFFER,
  FBO
};

//------------------------------------------------------------------------------
// Offscreen Buffer

class Buffer
{
public:

  Buffer() {}
  virtual ~Buffer() = 0;

  virtual void init(const Info &) throw (Error) = 0;
  virtual void bind() = 0;
  virtual void unbind() = 0;
  virtual void texture_bind() {}
  virtual const Info & info() const = 0;
  
private:

  Buffer(const Buffer &);
  Buffer & operator = (const Buffer &);
};

//------------------------------------------------------------------------------
// Conversion

Str & operator >> (Str &, BUFFER &) throw (Str);
Str & operator << (Str &, BUFFER);

//------------------------------------------------------------------------------
// Labels

const List<Str> & label_buffer();

//------------------------------------------------------------------------------
// Debugging

Dbg & operator << (Dbg &, BUFFER);

//------------------------------------------------------------------------------
// Global

class Global
{
public:
  
  Global() throw (Error);
  ~Global();

  void buffer(BUFFER);
  BUFFER buffer() const;
  Buffer * buffer_create() const;

private:

  BUFFER _buffer;
  
  class _Pbuffer;
  class _Fbo;
  class _Global;
  std::auto_ptr<_Global> _global;
};

Global * global();

}
}

#endif

