/*------------------------------------------------------------------------------
 lib/djv_image/io_pic.h

 Image I/O - PIC
 ---------------
 
 Softimage image file format.

 Support:
 
 * Read-only.
 * Image data:
   * RGB, RGBA, 8-bit.
   * RGB plus alpha, 8-bit.
 * File compression.

 References:
 
 * Softimage, "INFO: PIC file format",
   http://xsi.wiki.avid.com/index.php/INFO:_PIC_file_format

 Copyright (c) 2004-2008 Darby Johnston. All rights reserved.
 Use is subject to license terms.
------------------------------------------------------------------------------*/

#ifndef DJV_IMAGE_IO_PIC_H
#define DJV_IMAGE_IO_PIC_H

#include <djv_image/io.h>

namespace djv_image {
namespace io_pic {

//------------------------------------------------------------------------------
// Information

static const Str name = "PIC";

static const List<Str> extension_list = List<Str>() << ".pic";

//------------------------------------------------------------------------------
// Options

enum TYPE
{
  TYPE_RGB,
  TYPE_RGBA,
  TYPE_RGB_A
};

enum COMPRESS
{
  COMPRESS_NONE,
  COMPRESS_RLE
};

//------------------------------------------------------------------------------
// Load Plugin

class Load : public io::Load
{
public:

  Load();

  Plugin * copy() const;
  const Str & name() const;
  const List<Str> & extension_list() const;
  
  void open(const File &, Info *, Tag *, Time *) throw (Error);
  void open(const File &, List<Info> *, Tag *, Time *) throw (Error);
  const Image * get(int frame, int layer, data::PROXY) throw (Error);

private:

  void _open(const File &, Info *, Tag *) throw (Error);

  File _file;
  djv_base::Io _io;
  TYPE _type;
  bool _compress [2];
  Image _image;
  Data _tmp;
};

//------------------------------------------------------------------------------
// Functions

const uint8_t * rle_load(
  const uint8_t * in,
  const uint8_t * end,
  uint8_t * out,
  int size,
  int channels,
  int stride,
  bool endian
);

//------------------------------------------------------------------------------
// Labels

const List<Str> & label_compress();

}
}

#endif

