/*------------------------------------------------------------------------------
 include/djv_ui/callback_.h

 Copyright (c) 2004-2008 Darby Johnston. All rights reserved.
 Use is subject to license terms.
------------------------------------------------------------------------------*/

namespace djv_ui {

template <class T>
inline Signal<T>::Signal(Callback_Base * parent) : _parent(parent)
{
  if (_parent) _parent->signal_add(this);
}

template <class T>
inline Signal<T>::~Signal()
{
  del();

  if (_parent) _parent->signal_del(this);
}

template <class T>
inline void Signal<T>::set(Callback_Base * in, Callback * callback)
{
  in->connect_add(this);
  
  _list.push_back(Pair(in, (void *)callback));
}

template <class T>
inline void Signal<T>::del(Callback_Base * in)
{
  for (List<Pair>::iterator i = _list.begin(); i != _list.end(); ++i)
    if (in == i->first)
    {
      in->connect_del(this);
      _list.erase(i);
      break;
    }
}

template <class T>
inline void Signal<T>::del()
{
  for (uint i = 0; i < _list.size(); ++i)
    _list[i].first->connect_del(this);
  _list.clear();
}

template <class T>
inline void Signal<T>::emit(T in)
{
  for (uint i = 0; i < _list.size(); ++i)
    ((Callback *)_list[i].second)(_list[i].first, in);
}

template <class T>
inline void Signal<T>::release(Callback_Base * in)
{
  for (List<Pair>::iterator i = _list.begin(); i != _list.end(); ++i)
    if (in == i->first)
    {
      _list.erase(i);
      break;
    }
}

template <class T>
inline uint Signal<T>::size() const { return _list.size(); }

}

