/*------------------------------------------------------------------------------
 include/djv_ui/color.h
 
 Color Widgets
 -------------
 
 Copyright (c) 2004-2008 Darby Johnston. All rights reserved.
 Use is subject to license terms.
------------------------------------------------------------------------------*/

#ifndef DJV_UI_COLOR_H
#define DJV_UI_COLOR_H

#include <djv_image/color.h>
#include <djv_ui/widget.h>
#include <djv_ui/group.h>

namespace djv_ui {
namespace color {

//------------------------------------------------------------------------------
// Color Swatch

class Swatch : public group::Frame
{
public:

  Swatch();
  virtual ~Swatch();

  // Value.
  
  void set(const djv_image::Color &);
  const djv_image::Color & get() const;
  Signal<const djv_image::Color &> sig;
  
  // Widget.

  virtual int handle(int);
  virtual void draw();
  Signal<bool> push_sig;
  
  // Layout.

  virtual void dirty();
  
  // Debugging.
  
  virtual Str dbg_name() const { return "djv::ui::color::Swatch"; }
  
private:

  djv_image::Color _value;
};

class Swatch_Widget : public widget::Widget
{
public:

  Swatch_Widget();
  virtual ~Swatch_Widget();

  // Value.
  
  void set(const djv_image::Color &);
  const djv_image::Color & get() const;
  Signal<const djv_image::Color &> sig;
  
private:

  DJV_CALLBACK(Swatch_Widget, widget_callback, const djv_image::Color &);
  DJV_CALLBACK(Swatch_Widget, dialog_callback, bool);
  
  struct _Swatch_Widget;
  std::auto_ptr<_Swatch_Widget> _p;
};

//------------------------------------------------------------------------------
// Color Widget

class Widget : public widget::Widget
{
public:

  Widget();
  virtual ~Widget();
  
  // Value.
  
  void set(const djv_image::Color &);
  const djv_image::Color & get() const;
  Signal<const djv_image::Color &> sig;
  
  // Float range.
  
  void float_range(double min, double max);
  double float_min() const;
  double float_max() const;
  
  // Debugging.
  
  virtual Str dbg_name() const { return "djv::ui::color::Widget"; }
  
private:

  DJV_CALLBACK(Widget, int_callback, int);
  DJV_CALLBACK(Widget, float_callback, double);
  DJV_CALLBACK(Widget, format_callback, int);
  DJV_CALLBACK(Widget, type_callback, int);
  
  void widget_update();
  void value_update();

  djv_image::Color _value;
  double _float_min, _float_max;
  
  struct _Widget;
  std::auto_ptr<_Widget> _p;
};

//------------------------------------------------------------------------------
// Labels

extern const Str
  tooltip_red,
  tooltip_green,
  tooltip_blue,
  tooltip_alpha,
  tooltip_luminance;

}
}

#endif

