/*------------------------------------------------------------------------------
 include/djv_ui/num.h
 
 Numeric Value Widgets
 ---------------------

 Copyright (c) 2004-2008 Darby Johnston. All rights reserved.
 Use is subject to license terms.
------------------------------------------------------------------------------*/

#ifndef DJV_UI_NUM_H
#define DJV_UI_NUM_H

#include <djv_ui/widget.h>

namespace djv_ui {
namespace num {

//------------------------------------------------------------------------------
// Widget Base

template<class T>
class Base
{
public:

  inline Base(T min = T(), T max = T());
  inline virtual ~Base() = 0;

  // Value.

  inline virtual void set(T);
  inline const T & get() const;
  
  inline virtual void default_valid(bool);
  inline virtual void default_value(T);
  inline bool default_valid() const;
  inline const T & default_value() const;
  
  // Range.

  inline virtual void range(T min, T max);
  inline virtual void min_action();
  inline virtual void max_action();
  inline const T & min() const;
  inline const T & max() const;
  
  // Increment.

  inline virtual void inc(T inc_small, T inc_large);
  inline virtual void inc_small_action();
  inline virtual void inc_large_action();
  inline virtual void dec_small_action();
  inline virtual void dec_large_action();
  inline const T & inc_small() const;
  inline const T & inc_large() const;

  // Widget.
  
  inline int handle(int);

protected:

  inline void clamp(bool);
  
  virtual void value_callback() {}
  virtual void range_callback() {}

private:

  T _value;
  bool _default_valid;
  T _default_value;
  T _min, _max;
  bool _clamp;
  T _inc_small, _inc_large;
};

//------------------------------------------------------------------------------
// Integer Base

class Int_Base : public Base<int>
{
public:

  Int_Base(int min = -1000000, int max = 1000000);
};

//------------------------------------------------------------------------------
// Float Base

class Float_Base : public Base<double>
{
public:

  Float_Base(double min = -1000000.0, double max = 1000000.0);
  virtual ~Float_Base() {}

  // Value.
  
  virtual void precision(int);
  virtual int precision() const;
  
private:

  int _precision;
};

//------------------------------------------------------------------------------
// Increment Tool Widget

class Inc : public widget::Widget
{
public:

  Inc();
  virtual ~Inc();
  
  // Value.
  
  Signal<bool> inc_sig, dec_sig;

private:

  DJV_CALLBACK(Inc, inc_callback, bool);
  DJV_CALLBACK(Inc, dec_callback, bool);
  
  struct _Inc;
  std::auto_ptr<_Inc> _p;
};

}
}

#include <djv_ui/num_.h>

#endif

