/*------------------------------------------------------------------------------
 include/djv_ui/num_int.h
 
 Integer Number Widgets
 ----------------------

 Copyright (c) 2004-2008 Darby Johnston. All rights reserved.
 Use is subject to license terms.
------------------------------------------------------------------------------*/

#ifndef DJV_UI_NUM_INT_H
#define DJV_UI_NUM_INT_H

#include <djv_ui/num.h>

namespace djv_ui {
namespace num {

//------------------------------------------------------------------------------
// Integer Edit

class Int_Edit : public widget::Widget, public Int_Base
{
public:

  Int_Edit();
  virtual ~Int_Edit();
  
  // Value.
  
  Signal<int> sig;
  
  // Widget.
  
  virtual int handle(int);
  
  // Layout.
  
  virtual Item * group_align();
  
private:

  void value_callback();
  DJV_CALLBACK(Int_Edit, widget_callback, const Str &);
  DJV_CALLBACK(Int_Edit, inc_callback, bool);
  DJV_CALLBACK(Int_Edit, dec_callback, bool);
  
  void value_update();
  
  struct _Int_Edit;
  std::auto_ptr<_Int_Edit> _p;
};

//------------------------------------------------------------------------------
// Integer Display

class Int_Display : public widget::Widget
{
public:

  Int_Display();
  virtual ~Int_Display();

  // Value.
  
  void set(int);

private:

  struct _Int_Display;
  std::auto_ptr<_Int_Display> _p;
};

//------------------------------------------------------------------------------
// Integer Slider

class Int_Slider : public widget::Widget, public Int_Base
{
public:

  Int_Slider(
    int min = 0,
    int max = 100,
    const Str & label = Str()
  );
  virtual ~Int_Slider();
  
  // Value.
  
  virtual void default_valid(bool);
  bool default_valid() const;

  Signal<int> sig;
  
  // Label.
  
  void label(widget::Widget *);
  widget::Widget * label() const;
  virtual Item * label_align();
  
  // Widget.

  virtual void text_font(const Fl_Font &);
  virtual void text_color(const Fl_Color &);
  virtual const Fl_Font & text_font() const;
  virtual const Fl_Color & text_color() const;

  virtual int handle(int);
  
  // Layout.
  
  virtual Item * group_align();
  
private:
  
  void value_callback();
  void range_callback();
  DJV_CALLBACK(Int_Slider, widget_callback, const Str &);
  DJV_CALLBACK(Int_Slider, inc_callback, bool);
  DJV_CALLBACK(Int_Slider, dec_callback, bool);
  DJV_CALLBACK(Int_Slider, slider_callback, int);
  DJV_CALLBACK(Int_Slider, default_callback, bool);
  
  void value_update();
  void range_update();
  
  struct _Int_Slider;
  std::auto_ptr<_Int_Slider> _p;
};

//------------------------------------------------------------------------------
// Labels

extern const Str format_int;

}
}

#endif

