/*------------------------------------------------------------------------------
 lib/djv_base/mem.cxx

 Copyright (c) 2004-2008 Darby Johnston. All rights reserved.
 Use is subject to license terms.
------------------------------------------------------------------------------*/

#include <djv_base/mem.h>
#include <djv_base/str.h>
#include <string.h>
#ifndef DJV_MACOSX
#include <malloc.h>
#include <stdlib.h>
#endif

namespace djv_base {
namespace mem {

void * get(size_t in)
{
  // XXX

#if ! (defined(WIN32) || defined(DJV_MACOSX))
  return ::memalign(mem::align, in);
#else
  return ::malloc(in);
#endif
}

void del(void * in)
{
  ::free(in);
}

void copy(const void * in, void * out, size_t size)
{
  ::memcpy(out, in, size);
}

void set(uint8_t in, void * out, size_t size)
{
  ::memset(out, in, size);
}

void zero(void * out, size_t size)
{
  ::memset(out, 0, size);
}

int compare(const void * a, const void * b, size_t size)
{
  return ::memcmp(a, b, size);
}

const List<Str> & label_endian()
{
  static const List<Str> data = List<Str>() <<
    "MSB" <<
    "LSB";
  return data;
}

}
}

