/*------------------------------------------------------------------------------
 lib/djv_base/time.cxx

 Copyright (c) 2004-2008 Darby Johnston. All rights reserved.
 Use is subject to license terms.
------------------------------------------------------------------------------*/

#include <djv_base/str.h>
#include <djv_base/time.h>
#if defined(WIN32)
#include <windows.h>
#else
#include <sys/time.h>
#include <unistd.h>
#endif
#include <float.h>
#include <time.h>

namespace djv_base {
namespace time {

Timer::Timer()
{
#if defined(WIN32)
  //_t0 = _t1 = 0;
  _t0.QuadPart = 0;
  _t1.QuadPart = 0;
  ::QueryPerformanceFrequency(&_frequency);
#else
  _t0.tv_sec = _t0.tv_usec = 0;
  _t1.tv_sec = _t1.tv_usec = 0;
#endif

  start();
}

void Timer::start()
{
#if defined(WIN32)
  //_t0 = ::GetTickCount();
  ::QueryPerformanceCounter(&_t0);
#else
  ::gettimeofday(&_t0, 0);
#endif

  _t1 = _t0;
}

void Timer::check()
{
#if defined(WIN32)
  //_t1 = ::GetTickCount();
  ::QueryPerformanceCounter(&_t1);
#else
  ::gettimeofday(&_t1, 0);
#endif
}

#if ! defined(WIN32)

namespace {

inline void sub(const timeval & a, const timeval & b, timeval * out)
{
  out->tv_sec = a.tv_sec - b.tv_sec;
  out->tv_usec = a.tv_usec - b.tv_usec;
  
  if (out->tv_usec < 0)
  {
    --out->tv_sec;
    out->tv_usec += 1000000;
  }
}

}

#endif

double Timer::seconds() const
{
  double out = 0.0;

#if defined(WIN32)
  //out = (_t1 - _t0) / 1000.0;
  if (_frequency.QuadPart)
    out = (_t1.QuadPart - _t0.QuadPart) / double(_frequency.QuadPart);
#else
  timeval t;
  sub(_t1, _t0, &t);
  out = t.tv_sec + t.tv_usec / 1000000.0;
#endif
  
  return out;
}

double Timer::fps() const
{
  const double seconds = this->seconds();
  return (seconds != 0.0) ? (1.0 / seconds) : 0.0;
}

::time_t current()
{
  return ::time(0);
}

void sleep(double seconds)
{
#if defined(WIN32)
  ::Sleep(seconds * 1000);
#else
  ::sleep(seconds);
#endif
}

void seconds_to_time(double in, int * hour, int * minute, double * second)
{
  *hour = int(in) / (60 * 60);
  in -= *hour * 60 * 60;

  *minute = int(in) / 60;
  in -= *minute * 60;

  *second = in;
}

Str time_to_str(::time_t in)
{
  Str out;

//#if defined(WIN32)
//  char tmp [DJV_CSTR_SIZE];
//  ::ctime_s(tmp, DJV_CSTR_SIZE, &in);
//  out = tmp;
//#else
  out = ::ctime(&in);
//#endif

  // XXX Remove newline.

  if (out.size()) out.erase(out.size() - 1);

  return out;
}

Str label_time(double in)
{
  int hour = 0, minute = 0;
  double second = 0.0;
  djv_base::time::seconds_to_time(in, &hour, &minute, &second);

  return str::Format("%%:%%:%%").
    arg(hour, 2, '0').
    arg(minute, 2, '0').
    arg(second, 5, 2, '0');
}

}
}

