/*------------------------------------------------------------------------------
 lib/djv_image/gl_offscreen.h
 
 Copyright (c) 2004-2008 Darby Johnston. All rights reserved.
 Use is subject to license terms.
------------------------------------------------------------------------------*/

#ifndef _DJV_IMAGE_GL_OFFSCREEN_H
#define _DJV_IMAGE_GL_OFFSCREEN_H

#include <djv_image/gl_offscreen.h>

namespace djv_image {
namespace gl_offscreen {

//------------------------------------------------------------------------------
// Offscreen

class _Offscreen
{
public:

  virtual ~_Offscreen() = 0;

  virtual void init() throw (Error) = 0;
};

//------------------------------------------------------------------------------
// Pixel Buffer

class _Pbuffer;

class _Pbuffer_Factory
{
public:

  virtual ~_Pbuffer_Factory() = 0;
  
  virtual void init() throw (Error) = 0;
  virtual void add(_Pbuffer *) = 0;
  
  virtual _Pbuffer * get(const Info &) throw (Error) = 0;
};

class Global::_Pbuffer : public Buffer
{
public:

  _Pbuffer(_Pbuffer_Factory *);
  ~_Pbuffer();

  void init(const Info &) throw (Error);
  void bind();
  void unbind();
  const Info & info() const;
  
private:

  _Pbuffer(const _Pbuffer &);
  _Pbuffer & operator = (const _Pbuffer &);

  _Pbuffer_Factory * _factory;

  gl_offscreen::_Pbuffer * _p;
  
  class Context_Restore;
  std::auto_ptr<Context_Restore> _context_restore;
};

//------------------------------------------------------------------------------
// Frame Buffer Object

class _Fbo
{
public:

  _Fbo(const Info &);
  ~_Fbo();
  
  void bind();
  void unbind();
  void texture_bind();
  const Info & info() const { return _info; }
  
private:

  Info _info;
  GLuint _id, _restore;
  GLuint _texture;
};

class _Fbo_Factory
{
public:

  _Fbo_Factory();
  ~_Fbo_Factory();
  
  void init() throw (Error);
  void add(_Fbo *);
  _Fbo * get(const Info &) throw (Error);

private:

  List<_Fbo *> _list;
  static const int _list_max = 10;
};

class Global::_Fbo : public Buffer
{
public:

  _Fbo(_Fbo_Factory *);
  ~_Fbo();

  void init(const Info &) throw (Error);
  void bind();
  void unbind();
  void texture_bind();
  const Info & info() const;

private:

  _Fbo(const _Fbo &);
  _Fbo & operator = (const _Fbo &);

  _Fbo_Factory * _factory;
  gl_offscreen::_Fbo * _p;
};

//------------------------------------------------------------------------------
// Global

class Global::_Global
{
public:

  _Global() throw (Error);
  ~_Global();

  _Offscreen * offscreen() const { return _offscreen.get(); }
  _Pbuffer_Factory * pbuffer() const { return _pbuffer.get(); }
  _Fbo_Factory * fbo() const { return _fbo.get(); }
  
private:

  std::auto_ptr<_Offscreen> _offscreen;
  std::auto_ptr<_Pbuffer_Factory> _pbuffer;
  std::auto_ptr<_Fbo_Factory> _fbo;
};

}
}

#endif
