/*------------------------------------------------------------------------------
 lib/djv_image/io_ifl_load.cxx

 Copyright (c) 2004-2008 Darby Johnston. All rights reserved.
 Use is subject to license terms.
------------------------------------------------------------------------------*/

//#define DJV_DBG

#include <djv_image/io_ifl.h>
#include <djv_base/io.h>

namespace djv_image {
namespace io_ifl {

io::Plugin * Load::copy() const { return new Load; }

const Str & Load::name() const { return io_ifl::name; }

const List<Str> & Load::extension_list() const
  { return io_ifl::extension_list; }

void Load::open(const File & in, Info * info, Tag * tag, Time * time)
  throw (Error)
{
  DJV_DBG_OBJ("Load::open");
  DJV_DBG_PRN("in = " << in);
  
  _list.clear();

  List<Str> tmp = djv_base::io::lines(in);
  DJV_DBG_PRN("list = " << tmp);
  
  for (uint i = 0; i < tmp.size(); ++i)
  {
    if (tmp[i].empty()) continue;
    
    File file(tmp[i]);
    
    if (file.path.empty())
      file.path = in.path;
    
    if (file.seq_valid())
    {
      file.type = file::SEQ;
      _list += file::seq_expand(file);
    }
    else
    {
      _list += tmp[i];
    }
  }
  DJV_DBG_PRN("list = " << _list);

  _load = std::auto_ptr<io::Load>(io::global()->load(
    _list.size() ? _list[0] : Str(), info, tag, time));
  
  if (time)
  {
    time->list.resize(_list.size());
    for (uint i = 0; i < _list.size(); ++i)
      time->list[i] = i;
  }
}

void Load::open(const File & in, List<Info> * info, Tag * tag, Time * time)
  throw (Error)
{
  io::Load::open(in, info, tag, time);
}

const Image * Load::get(int frame, int layer, data::PROXY proxy) throw (Error)
{
  DJV_DBG_OBJ("Load::get");
  DJV_DBG_PRN("frame = " << frame);
  DJV_DBG_PRN("layer = " << layer);
  DJV_DBG_PRN("proxy = " << proxy);
  
  Str file;
  if (_list.size())
  {
    if (-1 == frame) file = _list[0];
    else if (frame >= 0 && frame < int(_list.size())) file = _list[frame];
  }
  DJV_DBG_PRN("file = " << file);

  Info info;
  _load = std::auto_ptr<io::Load>(io::global()->load(file, &info));
  return _load->get(-1, layer, proxy);
}

}
}
