/*------------------------------------------------------------------------------
 lib/djv_ui/image_prefs.cxx
 
 Copyright (c) 2004-2008 Darby Johnston. All rights reserved.
 Use is subject to license terms.
------------------------------------------------------------------------------*/

#undef DJV_DBG
//#define DJV_DBG

#include <djv_ui/app.h>
#include <djv_ui/image.h>
#include <djv_ui/prefs.h>
#include <djv_image/io.h>

namespace djv_ui {
namespace image {

namespace {

void _load(const prefs::Prefs & in, djv_image::io::Plugin * plugin)
{
  prefs::Prefs prefs(in, plugin->name());
  
  const List<Str> & option = plugin->option_list();
  for (uint i = 0; i < option.size(); ++i)
  {
    Str tmp;
    if (prefs.get(option[i], &tmp)) plugin->option(option[i], &tmp);
  }
}

void _save(prefs::Prefs * in, const djv_image::io::Plugin * plugin)
{
  prefs::Prefs prefs(in, plugin->name());
  
  const List<Str> & option = plugin->option_list();
  for (uint i = 0; i < option.size(); ++i)
    prefs.set(option[i], plugin->option(option[i]));
}

}

Prefs::Prefs()
{
  DJV_DBG_OBJ("Prefs::Prefs");
  
  prefs::Prefs prefs(prefs::prefs_global(), "image");
  
  djv_image::gl_data::Filter::FILTER filter =
    djv_image::gl_data::Filter::FILTER(0);
  djv_image::gl_data::Filter::Default filter_default;
  djv_image::gl_data::Filter::Custom filter_custom;
  djv_image::time::FPS fps(djv_image::time::FPS(0));
  djv_image::time::UNITS units(djv_image::time::UNITS(0));
  if (prefs::get(&prefs, "filter", &filter))
    djv_image::gl_data::global()->filter(filter);
  if (prefs::get(&prefs, "filter_default", &filter_default))
    djv_image::gl_data::global()->filter_default(filter_default);
  if (prefs::get(&prefs, "filter_custom", &filter_custom))
    djv_image::gl_data::global()->filter_custom(filter_custom);
  if (prefs::get(&prefs, "speed", &fps))
    djv_image::time::global()->speed(fps);
  if (prefs::get(&prefs, "units", &units))
    djv_image::time::global()->units(units);

  prefs::Prefs load(prefs, "load");
  const List<djv_image::io::Load *> & load_list =
    djv_image::io::global()->load()->list();
  for (uint i = 0; i < load_list.size(); ++i)
    _load(load, load_list[i]);
  
  prefs::Prefs save(prefs, "save");
  const List<djv_image::io::Save *> & save_list =
    djv_image::io::global()->save()->list();
  for (uint i = 0; i < save_list.size(); ++i)
    _load(save, save_list[i]);
}

Prefs::~Prefs()
{
  DJV_DBG_OBJ("Prefs::~Prefs");

  prefs::Prefs prefs(prefs::prefs_global(), "image");
  
  prefs::set(&prefs, "filter", djv_image::gl_data::global()->filter());
  prefs::set(&prefs, "filter_default",
    djv_image::gl_data::global()->filter_default());
  prefs::set(&prefs, "filter_custom",
    djv_image::gl_data::global()->filter_custom());
  prefs::set(&prefs, "speed", djv_image::time::global()->speed());
  prefs::set(&prefs, "units", djv_image::time::global()->units());

  prefs::Prefs load(prefs, "load");
  const List<djv_image::io::Load *> & load_list =
    djv_image::io::global()->load()->list();
  for (uint i = 0; i < load_list.size(); ++i)
    _save(&load, load_list[i]);
  
  prefs::Prefs save(prefs, "save");
  const List<djv_image::io::Save *> & save_list =
    djv_image::io::global()->save()->list();
  for (uint i = 0; i < save_list.size(); ++i)
    _save(&save, save_list[i]);
}

void Prefs::filter(djv_image::gl_data::Filter::FILTER in)
{
  if (in == filter()) return;
  
  djv_image::gl_data::global()->filter(in);
  
  djv_ui::app::global()->window_update();
}

djv_image::gl_data::Filter::FILTER Prefs::filter() const
{
  return djv_image::gl_data::global()->filter();
}

void Prefs::filter_default(const djv_image::gl_data::Filter::Default & in)
{
  if (in == filter_default()) return;
  
  djv_image::gl_data::global()->filter_default(in);
  
  djv_ui::app::global()->window_update();
}

const djv_image::gl_data::Filter::Default & Prefs::filter_default() const
{
  return djv_image::gl_data::global()->filter_default();
}

void Prefs::filter_custom(const djv_image::gl_data::Filter::Custom & in)
{
  if (in == filter_custom()) return;
  
  djv_image::gl_data::global()->filter_custom(in);
  
  djv_ui::app::global()->window_update();
}

const djv_image::gl_data::Filter::Custom & Prefs::filter_custom() const
{
  return djv_image::gl_data::global()->filter_custom();
}

void Prefs::speed(djv_image::time::FPS in)
{
  if (in == speed()) return;
  
  djv_image::time::global()->speed(in);
  
  djv_ui::app::global()->window_update();
}

djv_image::time::FPS Prefs::speed() const
{
  return djv_image::time::global()->speed();
}

void Prefs::units(djv_image::time::UNITS in)
{
  if (in == units()) return;
  
  djv_image::time::global()->units(in);
  
  djv_ui::app::global()->window_update();
}

djv_image::time::UNITS Prefs::units() const
{
  return djv_image::time::global()->units();
}

}
}
