/*------------------------------------------------------------------------------
 lib/djv_ui/menu_popup.cxx

 Copyright (c) 2004-2008 Darby Johnston. All rights reserved.
 Use is subject to license terms.
------------------------------------------------------------------------------*/

#include <djv_ui/icon.h>
#include <djv_ui/menu.h>
#include <djv_ui/style.h>
#include <FL/Fl.H>
#include <FL/Fl_Menu_Button.H>

namespace djv_ui {
namespace menu {

class Popup::Widget : public Fl_Menu_Button
{
public:

  Widget() :
    Fl_Menu_Button(0, 0, 0, 0)
  {}

  void draw() {}
};

Popup::Popup(const List<Str> & list) :
  sig(this),
  _icon(icon::get("tool_dec"))
{
  frame(FL_UP_BOX);

  Frame::add(_widget = new Widget);
  
  layout::Item::size(layout::FIXED, layout::FIXED);
  
  set(list);
  
  _widget->callback(widget_callback, this);
}

void Popup::set(const List<Str> & in)
{
  _group.del();
  for (uint i = 0; i < in.size(); ++i)
    _group.add(in[i]);

  mem::Buffer<Fl_Menu_Item> tmp;
  _group.fl(&tmp);
  _widget->clear();
  _widget->copy(tmp());
}

void Popup::set(const menu::Group & in)
{
  _group = in;
  
  mem::Buffer<Fl_Menu_Item> tmp;
  _group.fl(&tmp);
  _widget->clear();
  _widget->copy(tmp());
}

const menu::Group & Popup::get() const { return _group; }

void Popup::draw()
{
  Frame::draw();
  
  if (_icon)
    _icon->draw(x() + w() / 2 - _icon->w() / 2, y() + h() / 2 - _icon->h() / 2);

  if (Fl::focus() == this) draw_focus();
}
  
void Popup::dirty()
{
  Frame::dirty();

  _widget->textfont(text_font());
  _widget->textsize(style::global()->font_size());
  _widget->textcolor(text_color());
  
  size_hint(style::global()->size_tool_icon() + style::global()->margin() * 2);
}

void Popup::geom(const Box2i & in)
{
  Frame::geom(in);
  _widget->resize(in.x, in.y, in.w, in.h);
}

const Box2i & Popup::geom() const { return Frame::geom(); }

void Popup::widget_callback()
{
  sig.emit(_widget->value());
}

}
}

