/*------------------------------------------------------------------------------
 lib/djv_ui/prefs.cxx
 
 Copyright (c) 2004-2008 Darby Johnston. All rights reserved.
 Use is subject to license terms.
------------------------------------------------------------------------------*/

#undef DJV_DBG
//#define DJV_DBG

#include <djv_ui/prefs.h>
#include <FL/Fl_Preferences.H>
#include <stdlib.h>

namespace djv_ui {
namespace prefs {

Prefs::Prefs(const Str & name) :
  _prefs(new Fl_Preferences(
    Fl_Preferences::USER,
    Str(str::Format("djv-%%-%%").
      arg(DJV_VERSION_MAJOR).
      arg(DJV_VERSION_PREFS)).c_str(),
    name.c_str()))
{}

Prefs::Prefs(const Prefs & in, const Str & group) :
  _prefs(new Fl_Preferences(
    in._prefs.get(),
    group.c_str()))
{}

Prefs::Prefs(Prefs * in, const Str & group) :
  _prefs(new Fl_Preferences(
    in->_prefs.get(),
    group.c_str()))
{}

Prefs::~Prefs()
{
  _prefs->flush();
}

namespace {

Str pack_name(const Str & in)
{
  return str::replace(in, ':', '*');
}

Str unpack_name(const Str & in)
{
  return str::replace(in, '*', ':');
}

}

void Prefs::set(const Str & in, const Str & value)
{
  _prefs->set(pack_name(in).c_str(), value.c_str());
}

bool Prefs::get(const Str & in, Str * out) const
{
  char * buf = 0;
  const bool found =
    _prefs->get(pack_name(in).c_str(), buf, "") ? true : false;
  *out = Str(buf);
  if (buf) free(buf);
  return found;
}
  
void Prefs::del(const Str & in)
{
  _prefs->deleteEntry(pack_name(in).c_str());
}

void Prefs::del_group(const Str & in)
{
  _prefs->deleteGroup(pack_name(in).c_str());
}

List<Str> Prefs::list() const
{
  List<Str> out;
  const int size = _prefs->entries();
  for (int i = 0; i < size; ++i)
    out += unpack_name(_prefs->entry(i));
  return out;
}

List<Str> Prefs::group() const
{
  List<Str> out;
  const int size = _prefs->groups();
  for (int i = 0; i < size; ++i)
    out += unpack_name(_prefs->group(i));
  return out;
}

}
}

