/*------------------------------------------------------------------------------
 lib/djv_ui/prefs_dialog_widget.cxx

 Copyright (c) 2004-2008 Darby Johnston. All rights reserved.
 Use is subject to license terms.
------------------------------------------------------------------------------*/

#undef DJV_DBG
//#define DJV_DBG

#include <djv_ui/button.h>
#include <djv_ui/choice.h>
#include <djv_ui/color.h>
#include <djv_ui/group.h>
#include <djv_ui/image.h>
#include <djv_ui/label.h>
#include <djv_ui/layout_row.h>
#include <djv_ui/prefs_dialog.h>
#include <djv_ui/style.h>

namespace djv_ui {
namespace prefs_dialog {

Widget::Widget()
{
  // Widgets.

  group::Box * time_group =
    new group::Box(prefs_time_group);
  choice::Widget * time_units =
    new choice::Widget(djv_image::time::label_units());
  time_units->label(
    new label::Widget(prefs_time_units));
  choice::Widget * speed_widget =
    new choice::Widget(djv_image::time::label_fps(), prefs_speed);
  label::Widget * speed_unit_label =
    new label::Widget(prefs_speed_unit);

  group::Box * filter_group =
    new group::Box(prefs_filter_group);
  choice::Widget * filter =
    new choice::Widget(djv_image::gl_data::label_filter(), prefs_filter);
  choice::Widget * filter_default_min =
    new choice::Widget(djv_image::gl_data::label_filter_default());
  choice::Widget * filter_default_mag =
    new choice::Widget(djv_image::gl_data::label_filter_default());
  choice::Widget * filter_custom_min =
    new choice::Widget(djv_image::gl_data::label_filter_custom());
  choice::Widget * filter_custom_mag =
    new choice::Widget(djv_image::gl_data::label_filter_custom());
  label::Widget * filter_default_label =
    new label::Widget(prefs_filter_default);
  label::Widget * filter_custom_label =
    new label::Widget(prefs_filter_custom);
  label::Widget * filter_unit_label =
    new label::Widget(prefs_filter_unit);

  group::Box * style_group =
    new group::Box(prefs_style_group);
  choice::Widget * color =
    new choice::Widget(style::label_color(), prefs_color);

  label::Widget * color_custom_label =
    new label::Widget(prefs_color_custom);
  color::Swatch_Widget * color_foreground =
    new color::Swatch_Widget;
  label::Widget * color_foreground_label =
    new label::Widget(prefs_color_foreground);
  color::Swatch_Widget * color_background =
    new color::Swatch_Widget;
  label::Widget * color_background_label =
    new label::Widget(prefs_color_background);
  color::Swatch_Widget * color_background2 =
    new color::Swatch_Widget;
  label::Widget * color_background2_label =
    new label::Widget(prefs_color_background2);
  color::Swatch_Widget * color_select =
    new color::Swatch_Widget;
  label::Widget * color_select_label =
    new label::Widget(prefs_color_select);

  choice::Widget * size =
    new choice::Widget(style::label_size(), prefs_size);

  group::Box * misc_group =
    new group::Box(prefs_misc_group);
  button::Check * tooltip =
    new button::Check(prefs_tooltip);

  // Layout.
  
  layout::group(List<layout::Item *>() <<
    filter_default_label <<
    filter_custom_label);
  layout::group(List<layout::Item *>() <<
    filter_default_min <<
    filter_custom_min);
  layout::group(List<layout::Item *>() <<
    filter_default_mag <<
    filter_custom_mag);

  Layout_V * layout, * layout_group, * layout_v;
  Layout_H * layout_h;
  
  layout = new Layout_V(this);

  layout->add(time_group);
  layout_group = new Layout_V(time_group);
  layout_group->spacing(style::global()->spacing_large());
  layout_group->add(time_units);
  layout_h = new Layout_H(layout_group);
  layout_h->margin(0);
  layout_h->add(speed_widget);
  layout_h->add(speed_unit_label);

  layout->add(filter_group);
  layout_group = new Layout_V(filter_group);
  layout_group->spacing(style::global()->spacing_large());
  layout_group->add(filter);
  layout_v = new Layout_V(layout_group);
  layout_v->margin(0);
  layout_h = new Layout_H(layout_v);
  layout_h->margin(0);
  layout_h->add(filter_default_label);
  layout_h->add(filter_default_min);
  layout_h->add(filter_default_mag);
  layout_h->add(filter_unit_label);
  layout_h = new Layout_H(layout_v);
  layout_h->margin(0);
  layout_h->add(filter_custom_label);
  layout_h->add(filter_custom_min);
  layout_h->add(filter_custom_mag);

  layout->add(style_group);
  layout_group = new Layout_V(style_group);
  layout_group->spacing(style::global()->spacing_large());
  layout_group->add(color);
  layout_v = new Layout_V(layout_group);
  layout_v->margin(0);
  layout_v->add(color_custom_label);
  layout_h = new Layout_H(layout_v);
  layout_h->margin(0);
  layout_h->add(color_foreground);
  layout_h->add(color_foreground_label);
  layout_h = new Layout_H(layout_v);
  layout_h->margin(0);
  layout_h->add(color_background);
  layout_h->add(color_background_label);
  layout_h = new Layout_H(layout_v);
  layout_h->margin(0);
  layout_h->add(color_background2);
  layout_h->add(color_background2_label);
  layout_h = new Layout_H(layout_v);
  layout_h->margin(0);
  layout_h->add(color_select);
  layout_h->add(color_select_label);
  layout_group->add(size);

  layout->add(misc_group);
  layout_group = new Layout_V(misc_group);
  layout_group->spacing(style::global()->spacing_large());
  layout_group->add(tooltip);

  layout->add_stretch();
  
  // Initialize.
  
  time_units->set(image::prefs()->units());
  speed_widget->set(djv_image::time::global()->speed());
  filter->set(image::prefs()->filter());
  filter_default_min->set(image::prefs()->filter_default().first);
  filter_default_mag->set(image::prefs()->filter_default().second);
  filter_custom_min->set(image::prefs()->filter_custom().first);
  filter_custom_mag->set(image::prefs()->filter_custom().second);
  color->set(style::prefs()->color());
  color_foreground->set(style::prefs()->color_custom().foreground);
  color_background->set(style::prefs()->color_custom().background);
  color_background2->set(style::prefs()->color_custom().background2);
  color_select->set(style::prefs()->color_custom().select);
  size->set(style::prefs()->size());
  tooltip->set(base::prefs()->tooltip());

  // Callbacks.

  time_units->sig.set(this, time_units_callback);
  speed_widget->sig.set(this, speed_callback);
  filter->sig.set(this, filter_callback);
  filter_default_min->sig.set(this, filter_default_min_callback);
  filter_default_mag->sig.set(this, filter_default_mag_callback);
  filter_custom_min->sig.set(this, filter_custom_min_callback);
  filter_custom_mag->sig.set(this, filter_custom_mag_callback);
  color->sig.set(this, color_callback);
  color_foreground->sig.set(this, color_foreground_callback);
  color_background->sig.set(this, color_background_callback);
  color_background2->sig.set(this, color_background2_callback);
  color_select->sig.set(this, color_select_callback);
  size->sig.set(this, size_callback);
  tooltip->sig.set(this, tooltip_callback);
}

void Widget::time_units_callback(int in)
  { image::prefs()->units(djv_image::time::UNITS(in)); }

void Widget::speed_callback(int in)
  { djv_image::time::global()->speed(djv_image::time::FPS(in)); }

void Widget::filter_callback(int in)
  { image::prefs()->filter(djv_image::gl_data::Filter::FILTER(in)); }

void Widget::filter_default_min_callback(int in)
{
  using namespace djv_image::gl_data;
  
  Filter::Default value = image::prefs()->filter_default();
  value.first = Filter::DEFAULT(in);
  image::prefs()->filter_default(value);
}

void Widget::filter_default_mag_callback(int in)
{
  using namespace djv_image::gl_data;
  
  Filter::Default value = image::prefs()->filter_default();
  value.second = Filter::DEFAULT(in);
  image::prefs()->filter_default(value);
}

void Widget::filter_custom_min_callback(int in)
{
  using namespace djv_image::gl_data;
  
  Filter::Custom value = image::prefs()->filter_custom();
  value.first = Filter::CUSTOM(in);
  image::prefs()->filter_custom(value);
}

void Widget::filter_custom_mag_callback(int in)
{
  using namespace djv_image::gl_data;
  
  Filter::Custom value = image::prefs()->filter_custom();
  value.second = Filter::CUSTOM(in);
  image::prefs()->filter_custom(value);
}

void Widget::color_callback(int in)
  { style::prefs()->color(style::COLOR(in)); }

void Widget::color_foreground_callback(const djv_image::Color & in)
{
  style::Color tmp = style::prefs()->color_custom();
  tmp.foreground = in;
  style::prefs()->color_custom(tmp);
}

void Widget::color_background_callback(const djv_image::Color & in)
{
  style::Color tmp = style::prefs()->color_custom();
  tmp.background = in;
  style::prefs()->color_custom(tmp);
}

void Widget::color_background2_callback(const djv_image::Color & in)
{
  style::Color tmp = style::prefs()->color_custom();
  tmp.background2 = in;
  style::prefs()->color_custom(tmp);
}

void Widget::color_select_callback(const djv_image::Color & in)
{
  style::Color tmp = style::prefs()->color_custom();
  tmp.select = in;
  style::prefs()->color_custom(tmp);
}

void Widget::size_callback(int in) { style::prefs()->size(style::SIZE(in)); }

void Widget::tooltip_callback(bool in) { base::prefs()->tooltip(in); }

}
}

