/*------------------------------------------------------------------------------
 lib/djv_ui/text_edit.cxx

 Copyright (c) 2004-2008 Darby Johnston. All rights reserved.
 Use is subject to license terms.
------------------------------------------------------------------------------*/

#undef DJV_DBG
//#define DJV_DBG

#include <djv_ui/style.h>
#include <djv_ui/text.h>
#include <FL/Fl.H>
#include <FL/Fl_Group.H>
#include <FL/Fl_Input.H>
#include <FL/fl_draw.H>

namespace djv_ui {
namespace text {

namespace {

class _Input : public Fl_Input
{
public:

  _Input() :
    Fl_Input(0, 0, 0, 0)
  {
    when(FL_WHEN_CHANGED);
    box(FL_FLAT_BOX);
  }
};

}

struct Edit::_Edit
{
  _Input * widget;
  Fl_Color color; // XXX Used to return a reference in get method.
};

Edit::Edit(const Str & in) :
  sig(this),
  action_sig(this),
  change_sig(this),
  _value(in),
  _p(new _Edit)
{
  frame(FL_DOWN_FRAME);
  
  add(_p->widget = new _Input);

  Item::size(layout::STRETCH, layout::FIXED);

  widget_update();

  _p->widget->callback(widget_callback, this);
}

Edit::~Edit()
{}

void Edit::set(const Str & in)
{
  if (in == _value) return;
  
  DJV_DBG_OBJ("Edit::set");
  DJV_DBG_PRN("in = " << in);
  
  _value = in;
  
  widget_update();
  sig.emit(_value);
}

const Str & Edit::get() const { return _value; }

void Edit::text_font(const Fl_Font & in)
{
  Frame::text_font(in);
  _p->widget->textfont(in);
}

void Edit::color(const Fl_Color & in)
{
  Frame::color(in);
  _p->widget->color(in);
}

const Fl_Color & Edit::color() const { return _p->color = Frame::color(); }
  
const Fl_Font & Edit::text_font() const { return Frame::text_font(); }

void Edit::text_color(const Fl_Color & in)
{
  Frame::text_color(in);
  _p->widget->textcolor(in);
}

const Fl_Color & Edit::text_color() const { return Frame::text_color(); }

int Edit::handle(int in)
{
  switch (in)
  {
    case FL_KEYBOARD:
      if (! Fl::event_state(FL_SHIFT | FL_CTRL | FL_ALT))
        switch (Fl::event_key())
        {
          case FL_Enter:
          case FL_KP_Enter:
            sig.emit(_p->widget->value());
            action_sig.emit(_p->widget->value());
            return 1;
        }
      break;
      
    case FL_UNFOCUS:
      sig.emit(_p->widget->value());
      break;
  }
  return Frame::handle(in);
}

void Edit::size_str(const Str & in)
{
  _size_str = in;
}
  
const Str & Edit::size_str() const { return _size_str; }

void Edit::dirty()
{
  Frame::dirty();

  _p->widget->textfont(text_font());
  _p->widget->textsize(style::global()->font_size());
  _p->widget->textcolor(text_color());
  
  fl_font(text_font(), style::global()->font_size());
  size_hint(
    Vec2i(
      math::max(style::global()->size_button(), base::str_width(_size_str)),
      fl_height()) +
    style::global()->margin_text() * 2 + frame_size() * 2);
}

void Edit::geom(const Box2i & in)
{
  Frame::geom(in);
  const Box2i & geom = frame_geom();
  _p->widget->resize(geom.x, geom.y, geom.w, geom.h);
}
  
const Box2i & Edit::geom() const { return Frame::geom(); }

void Edit::widget_callback()
{
  _value = _p->widget->value();
  change_sig.emit(_value);
}

void Edit::widget_update()
{
  _p->widget->value(_value.c_str());
  //_p->widget->position(0, 0);
  _p->widget->position(_value.size());
}

}
}
