/*------------------------------------------------------------------------------
 plugin/djv_jpeg/ui.cxx

 Copyright (c) 2004-2008 Darby Johnston. All rights reserved.
 Use is subject to license terms.
------------------------------------------------------------------------------*/

#include "ui.h"
#include <djv_ui/group.h>
#include <djv_ui/label.h>
#include <djv_ui/layout_row.h>
#include <djv_ui/num_int.h>
#include <djv_ui/style.h>

namespace djv_jpeg {
namespace ui {

extern "C" {
DJV_PLUGIN_EXPORT plugin::Plugin * djv_ui_image_save() { return new Save; }
}

struct Save::_Save
{
  num::Int_Slider * quality;
};

Save::Save() :
  _plugin(0),
  _p(new _Save)
{
  // Widgets.
  
  group::Box * quality_group =
    new group::Box(label_quality_group);
  _p->quality =
    new num::Int_Slider(0, 100, label_quality);
  
  // Layout.
  
  Layout_V * layout, * layout_group;
  
  layout = new Layout_V(this);
  
  layout->add(quality_group);
  layout_group = new Layout_V(quality_group);
  layout_group->spacing(style::global()->spacing_large());
  layout_group->add(_p->quality);
  
  layout->add_stretch();
  
  // Initialize.
  
  plugin_update();
  widget_update();
  
  // Callbacks.
  
  _p->quality->sig.set(this, quality_callback);
}

plugin::Plugin * Save::copy() const { return new Save; }

const Str & Save::name() const { return djv_jpeg::name; }

void Save::set(djv_image::io::Plugin * in)
{
  _plugin = in;
  
  widget_update();
}

void Save::quality_callback(int in)
{
  if (! _plugin) return;
  
  _option.quality = in;
  
  plugin_update();
  sig.emit(_plugin);
}

void Save::plugin_update()
{
  callbacks(false);

  if (_plugin)
  {
    Str tmp;
    tmp << _option.quality;
    _plugin->option(_plugin->option_list()[io::SAVE_QUALITY], &tmp);
  }

  callbacks(true);
}

void Save::widget_update()
{
  callbacks(false);
  
try
{
  if (_plugin)
  {
    Str tmp;
    tmp = _plugin->option(_plugin->option_list()[io::SAVE_QUALITY]);
    tmp >> _option.quality;
  }
}
catch (Str) {}
  
  _p->quality->set(_option.quality);
  
  callbacks(true);
}

const Str
  label_quality_group = "Quality",
  label_quality = "Value:";

}
}

