/*------------------------------------------------------------------------------
 plugin/djv_quicktime/etc.cxx
 
 Copyright (c) 2004-2008 Darby Johnston. All rights reserved.
 Use is subject to license terms.
------------------------------------------------------------------------------*/

//#define DJV_DBG

#include "base.h"
namespace qt {
#if defined(DJV_MACOSX)
#include <QuickTime/QTML.h>
#elif defined(WIN32)
#include <QTML.h>
#endif
}

namespace djv_quicktime {

Handle::Handle(qt::Handle in) :
  p(in)
{}

Handle::~Handle()
{
  if (p) qt::DisposeHandle(p);
}

void Handle::init(long in) throw (Error)
{
  p = qt::NewHandle(in);
  if (! p) throw Error(name, error_handle);
}

CFStringRef::CFStringRef(const Str & in) :
  p(qt::CFStringCreateWithCString(0, in.c_str(), 0))
{}
  
CFStringRef::~CFStringRef()
{ 
  qt::CFRelease(p);
}

namespace { int _ref = 0; }

void quicktime_init() throw (Error)
{
  ++_ref;
  if (_ref > 1) return;

  DJV_DBG_OBJ("quicktime_init");

  qt::OSErr err = qt::noErr;

#if defined(WIN32)
  err = qt::InitializeQTML(0); //qt::kInitializeQTMLNoSoundFlag);
  if (err != qt::noErr)
    throw Error(name, str::Format(error_init).arg(err));
#endif
 
  long version = 0;
  err = qt::Gestalt(qt::gestaltQuickTime, &version);
  if (err != qt::noErr)
    throw Error(name, str::Format(error_version).arg(err));
  
  err = qt::EnterMovies();
  if (err != qt::noErr)
    throw Error(name, str::Format(error_init).arg(err));
}

void quicktime_del()
{
  --_ref;
  if (_ref) return;
  
  DJV_DBG_OBJ("quicktime_del");

  qt::ExitMovies();

#if defined(WIN32)
  qt::TerminateQTML();
#endif
}

qt::CodecType quicktime_codec(CODEC in)
{
  switch (in)
  {
    case CODEC_RAW: return qt::kRawCodecType;
    case CODEC_JPEG: return qt::kJPEGCodecType;
    case CODEC_MJPEGA: return qt::kMotionJPEGACodecType;
    case CODEC_MJPEGB: return qt::kMotionJPEGBCodecType;
    case CODEC_H263: return qt::kH263CodecType;
    case CODEC_H264: return qt::kH264CodecType;
    case CODEC_DVCNTSC: return qt::kDVCNTSCCodecType;
    case CODEC_DVCPAL: return qt::kDVCPALCodecType;
  }
  return 0;
}

qt::CodecQ quicktime_quality(QUALITY in)
{
  switch (in)
  {
    case QUALITY_LOSSLESS: return qt::codecLosslessQuality;
    case QUALITY_MIN: return qt::codecMinQuality;
    case QUALITY_MAX: return qt::codecMaxQuality;
    case QUALITY_LOW: return qt::codecLowQuality;
    case QUALITY_NORMAL: return qt::codecNormalQuality;
    case QUALITY_HIGH: return qt::codecHighQuality;
  }
  return 0;
}

DJV_STR_OPERATOR_SET(CODEC, label_codec())
DJV_STR_OPERATOR_GET(CODEC, label_codec())
DJV_STR_OPERATOR_SET(QUALITY, label_quality())
DJV_STR_OPERATOR_GET(QUALITY, label_quality())

const List<Str> & label_codec()
{
  static const List<Str> data = List<Str>() <<
    "Raw" <<
    "JPEG" <<
    "MJPEG-A" <<
    "MJPEG-B" <<
    "H263" <<
    "H264" <<
    "DVC-NTSC" <<
    "DVC-PAL";
  return data;
}

const List<Str> & label_quality()
{
  static const List<Str> data = List<Str>() <<
    "Lossless" <<
    "Min" <<
    "Max" <<
    "Low" <<
    "Normal" <<
    "High";
  return data;
}

const Str
  error_init = "Initialize = %%",
  error_version = "Version = %%",
  error_open = "Open = %%",
  error_gworld = "GWorld = %%",
  error_track = "Track = %%",
  error_media = "Media = %%",
  error_media_begin = "Media Begin = %%",
  error_media_end = "Media End = %%",
  error_media_add = "Media Add = %%",
  error_media_sample = "Media Sample = %%",
  error_movie_add = "Movie Add = %%",
  error_compress = "Compress = %%",
  error_handle = "Handle = %%";

}

