/*------------------------------------------------------------------------------
 plugin/djv_quicktime/load.cxx

 Copyright (c) 2004-2008 Darby Johnston. All rights reserved.
 Use is subject to license terms.
------------------------------------------------------------------------------*/

//#define DJV_DBG

#include "io.h"
namespace qt {
#if defined(WIN32)
#include <CFString.h>
#include <ImageCompression.h>
#endif
}

namespace djv_quicktime {
namespace io {

extern "C" {
DJV_PLUGIN_EXPORT plugin::Plugin * djv_image_load() { return new Load; }
}

Load::Load() :
  _movie(0),
  _gworld(0)
{}

Load::~Load()
{
  _close();
}

void Load::init() throw (Error)
{
  quicktime_init();
}

void Load::del()
{
  quicktime_del();
}

plugin::Plugin * Load::copy() const { return new Load; }

const Str & Load::name() const { return djv_quicktime::name; }

const List<Str> & Load::extension_list() const
  { return djv_quicktime::extension_list; }

namespace {

const qt::OSType media_list [] = { qt::VisualMediaCharacteristic };
const int media_list_size = 1;

/*long _frame_count(qt::Movie in)
{
  long out = 0;  
  for (qt::TimeValue time = 0; time >= 0; ++out)
  {
    qt::TimeValue tmp = 0;
    qt::GetMovieNextInterestingTime(in, qt::nextTimeStep, media_list_size,
      media_list, time, qt_fixed1, &tmp, 0);
    time = tmp;
  }
  return out;
}*/

qt::TimeValue _frame_duration(qt::Movie in)
{
  qt::TimeValue out = 0;
  qt::GetMovieNextInterestingTime(in, qt::nextTimeStep, media_list_size,
    media_list, out, qt_fixed1, 0, &out);
  return out;
}

}

void Load::open(const File & in, Info * info, Tag * tag, djv_image::Time * time)
  throw (Error)
{
  DJV_DBG_OBJ("Load::open");
  DJV_DBG_PRN("in = " << in);

  _close();

  // Open.

  CFStringRef cf_str_ref(file::path_absolute(in));

  qt::QTNewMoviePropertyElement prop_list [10];
  qt::ItemCount prop_size = 0;

  prop_list[prop_size].propClass = qt::kQTPropertyClass_DataLocation;
  prop_list[prop_size].propID =
    qt::kQTDataLocationPropertyID_CFStringNativePath;
    //qt::kQTDataLocationPropertyID_CFStringPosixPath;
  prop_list[prop_size].propValueSize = sizeof(cf_str_ref.p);
  prop_list[prop_size].propValueAddress = (void *)&cf_str_ref.p;
  prop_list[prop_size].propStatus = 0;
  ++prop_size;

  qt::Boolean unresolved = true;
  prop_list[prop_size].propClass = qt::kQTPropertyClass_MovieInstantiation;
  prop_list[prop_size].propID =
    qt::kQTMovieInstantiationPropertyID_DontAskUnresolvedDataRefs;
  prop_list[prop_size].propValueSize = sizeof(unresolved);
  prop_list[prop_size].propValueAddress = &unresolved;
  prop_list[prop_size].propStatus = 0;
  ++prop_size;

  qt::Boolean active = true;
  prop_list[prop_size].propClass = qt::kQTPropertyClass_NewMovieProperty;
  prop_list[prop_size].propID = qt::kQTNewMoviePropertyID_Active;
  prop_list[prop_size].propValueSize = sizeof(active);
  prop_list[prop_size].propValueAddress = &active;
  prop_list[prop_size].propStatus = 0;
  ++prop_size;

  qt::Boolean no_interact = true;
  prop_list[prop_size].propClass = qt::kQTPropertyClass_NewMovieProperty;
  prop_list[prop_size].propID = qt::kQTNewMoviePropertyID_DontInteractWithUser;
  prop_list[prop_size].propValueSize = sizeof(no_interact);
  prop_list[prop_size].propValueAddress = &no_interact;
  prop_list[prop_size].propStatus = 0;
  ++prop_size;

  qt::QTVisualContextRef visual = 0;
  prop_list[prop_size].propClass = qt::kQTPropertyClass_Context;
  prop_list[prop_size].propID = qt::kQTContextPropertyID_VisualContext;
  prop_list[prop_size].propValueSize = sizeof(qt::QTVisualContextRef);
  prop_list[prop_size].propValueAddress = &visual;
  prop_list[prop_size].propStatus = 0;
  ++prop_size;

  qt::QTAudioContextRef audio = 0;
  prop_list[prop_size].propClass = qt::kQTPropertyClass_Context;
  prop_list[prop_size].propID = qt::kQTContextPropertyID_AudioContext;
  prop_list[prop_size].propValueSize = sizeof(qt::QTAudioContextRef);
  prop_list[prop_size].propValueAddress = &audio;
  prop_list[prop_size].propStatus = 0;
  ++prop_size;

  qt::OSErr err = qt::NewMovieFromProperties(prop_size, prop_list, 0, 0,
    &_movie);
  if (err != qt::noErr)
    throw Error(name(), str::Format(error_open).arg(err));
  
  // Information.
  
  _info.name = in;
  
  qt::Rect rect;
  qt::GetMovieBox(_movie, &rect);
  _info.size = Vec2i(rect.right - rect.left, rect.bottom - rect.top);
  _info.mirror.y = true;
  _info.pixel = pixel::RGBA_U8;
  //_info.bgr = true;
 
  const qt::TimeScale time_scale = qt::GetMovieTimeScale(_movie);
  const qt::TimeValue time_duration = qt::GetMovieDuration(_movie);
  //const long frame_count = _frame_count(_movie);
  const qt::TimeValue frame_duration = _frame_duration(_movie);
  //const TimeValue frame_duration = time_duration / frame_count;
  DJV_DBG_PRN("time scale = " << int(time_scale));
  DJV_DBG_PRN("time duration = " << int(time_duration));
  //DJV_DBG_PRN("frame count = " << int(frame_count));
  DJV_DBG_PRN("frame duration = " << int(frame_duration));

  _time = djv_image::Time(
    0, frame_duration ? (time_duration / frame_duration - 1) : 0,
    djv_image::time::Speed(time_scale, frame_duration));
  DJV_DBG_PRN("time = " << _time);
  
  *info = _info;
  *time = _time;
  
  // Temporary buffers.
  
  _image.set(_info);
  
  // XXX

  unsigned long qt_format =
#if defined(DJV_MACOSX)
    qt::k32ARGBPixelFormat;
#else
    qt::k32RGBAPixelFormat;
#endif

  err = qt::NewGWorldFromPtr(
    &_gworld,
    qt_format,
    &rect,
    0,
    0,
    0,
    (char *)_image.data(0, 0),
    _info.size.x * 4
  );
  if (err != qt::noErr)
    throw Error(name(), str::Format(error_gworld).arg(err));

  qt::SetMovieGWorld(_movie, _gworld, 0);
}

void Load::open(const File & in, List<Info> * info, Tag * tag,
  djv_image::Time * time) throw (Error)
{
  djv_image::io::Load::open(in, info, tag, time);
}

void Load::_close()
{
  DJV_DBG_OBJ("Load::_close");
  
  if (_movie) { qt::DisposeMovie(_movie); _movie = 0; }
  if (_gworld) { qt::DisposeGWorld(_gworld); _gworld = 0; }
}

const Image * Load::get(int frame, int, data::PROXY proxy) throw (Error)
{
  DJV_DBG_OBJ("Load::get");
  DJV_DBG_PRN("frame = " << frame);
  DJV_DBG_PRN("proxy = " << proxy);

  if (-1 == frame) frame = 0;

  // Read.
  
  const qt::TimeValue time = frame * _time.speed.duration;
  DJV_DBG_PRN("time = " << int(time));
  qt::SetMovieTimeValue(_movie, time);
  qt::MoviesTask(_movie, 0);
  qt::UpdateMovie(_movie);
  
  Info info = _info;
  if (proxy)
  {
    info.size = data::proxy_scale(info.size, proxy);
    info.proxy = proxy;
    _tmp.set(info);
    data::proxy_scale(_image, &_tmp, proxy);
  }
 
  // XXX ARGB -> RGB

#if defined(DJV_MACOSX)
  uint32_t * p = (uint32_t *)(proxy ? _tmp.data() : _image.data());
  const uint32_t * const end = p + info.size.x * info.size.y;
  for (; p < end; ++p)
    *p = (*p) << 8;
#endif

  DJV_DBG_PRN("image = " << _image);
  return proxy ? &_tmp : &_image;
}

}
}

