/*------------------------------------------------------------------------------
 plugin/djv_quicktime/save.cxx

 Copyright (c) 2004-2008 Darby Johnston. All rights reserved.
 Use is subject to license terms.
------------------------------------------------------------------------------*/

//#define DJV_DBG

#include "io.h"
namespace qt {
#if defined(DJV_MACOSX)
#elif defined(WIN32)
#include <ImageCompression.h>
#include <QTML.h>
#endif
#define qt_anyCodec ((qt::CodecComponent)0) // XXX
}

namespace djv_quicktime {
namespace io {

extern "C" {
DJV_PLUGIN_EXPORT plugin::Plugin * djv_image_save() { return new Save; }
}

Save::Save() :
  _frame_duration(0),
  _f(0),
  _movie(0),
  _track(0),
  _media(0),
  _gworld(0)
{}

void Save::init() throw (Error)
{
  quicktime_init();
}

void Save::del()
{
  quicktime_del();
}

plugin::Plugin * Save::copy() const
{
  Save * out = new Save;
  out->_option = _option;
  return out;
}

const Str & Save::name() const { return djv_quicktime::name; }

const List<Str> & Save::extension_list() const
  { return djv_quicktime::extension_list; }

bool Save::option(const Str & in, Str * data)
{
try
{
  if (str::compare(in, option_list()[SAVE_CODEC])) *data >> _option.codec;
  else if (str::compare(in, option_list()[SAVE_QUALITY]))
    *data >> _option.quality;
}
catch (Str) { return false; }
  return true;
}

Str Save::option(const Str & in) const
{
  Str out;
  if (str::compare(in, option_list()[SAVE_CODEC])) out << _option.codec;
  else if (str::compare(in, option_list()[SAVE_QUALITY]))
    out << _option.quality;
  return out;
}

const List<Str> & Save::option_list() const { return label_save_option(); }

void Save::open(const File & in, const Info & info, const Tag &,
  const djv_image::Time & time) throw (Error)
{
  DJV_DBG_OBJ("Save::open");
  DJV_DBG_PRN("in = " << in);
  DJV_DBG_PRN("info = " << info);
  DJV_DBG_PRN("time = " << time);
  
  _open(in, info);
  
  _track = qt::NewMovieTrack(
    _movie,
    qt::FixRatio(info.size.x, 1),
    qt::FixRatio(info.size.y, 1),
    qt::kNoVolume);
  if (! _track)
    throw Error(name(), str::Format(error_track).arg(qt::GetMoviesError()));
  
  _media = qt::NewTrackMedia(
    _track,
    qt::VideoMediaType,
    time.speed.scale,
    0,
    0);
  if (! _media)
    throw Error(name(), str::Format(error_media).arg(qt::GetMoviesError()));
  
  qt::OSErr err = qt::BeginMediaEdits(_media);
  if (err != qt::noErr)
    throw Error(name(), str::Format(error_media_begin).arg(err));
  
  // Information.

  _frame_duration = time.speed.duration;

  // Temporary buffers.
  
  Info _info;
  _info.size = info.size;
  _info.mirror.y = true;
  _info.pixel = pixel::RGBA_U8;
  _image.set(_info);
  
  const qt::Rect rect = { 0, 0, _info.size.y, _info.size.x };
  err = qt::NewGWorldFromPtr(
    &_gworld,
    qt::k32RGBAPixelFormat,
    &rect,
    0,
    0,
    0,
    (char *)_image.data(0, 0),
    _info.size.x * 4);
  if (err != qt::noErr)
    throw Error(name(), str::Format(error_gworld).arg(err));

  qt::SetMovieGWorld(_movie, _gworld, 0);
}

void Save::_open(const File & in, const Info & info) throw (Error)
{
  DJV_DBG_OBJ("Save::_open");
  DJV_DBG_PRN("in = " << in);
  DJV_DBG_PRN("info = " << info);

  /*qt::FSSpec file_spec;
#if defined(DJV_MACOSX)
  qt::Str255 file_name;
  qt::c2pstrcpy(file_name, file::path_absolute(in.get()).c_str());
  qt::FSMakeFSSpec(0, 0, file_name, &file_spec);
#elif defined(WIN32)
  char file_name [str::cstr_size];
  str::cstr(file::path_absolute(in), file_name, str::cstr_size);
  qt::NativePathNameToFSSpec(file_name, &file_spec, 0);
#endif

  qt::OSErr err = qt::CreateMovieFile(
    &file_spec,
    FOUR_CHAR_CODE('TVOD'),
    qt::smCurrentScript,
    qt::createMovieFileDeleteCurFile | qt::createMovieFileDontCreateResFile,
    &_f,
    &_movie);
  if (err != qt::noErr)
    throw Error(name(), str::Format(error_open).arg(err));*/

  Handle handle;
  qt::OSType handle_type;
  qt::OSErr err = qt::QTNewDataReferenceFromFullPathCFString(
    CFStringRef(file::path_absolute(in)).p,
    qt::kQTNativeDefaultPathStyle,
    //qt::kQTPOSIXPathStyle,
    0,
    &handle.p,
    &handle_type);
  if (err != qt::noErr)
    throw Error(name(), "poo!");

  err = qt::CreateMovieStorage(
    handle.p,
    handle_type,
    FOUR_CHAR_CODE('TVOD'),
    qt::smCurrentScript,
    qt::createMovieFileDeleteCurFile | qt::createMovieFileDontCreateResFile,
    &_f,
    &_movie);
  if (err != qt::noErr)
    throw Error(name(), str::Format(error_open).arg(err));
}

void Save::close() throw (Error)
{
  DJV_DBG_OBJ("Save::close");
  
  if (_track && _media)
  {
    DJV_DBG_PRN("end");

    qt::OSErr err = qt::EndMediaEdits(_media);
    if (err != qt::noErr)
      throw Error(name(), str::Format(error_media_end).arg(err));
    
    err = qt::InsertMediaIntoTrack(
      _track,
      0,
      0,
      qt::GetMediaDuration(_media),
      qt_fixed1);
    if (err != qt::noErr)
      throw Error(name(), str::Format(error_media_add).arg(err));
    
    /*short id = qt::movieInDataForkResID;
    err = qt::AddMovieResource(_movie, _f, &id, 0);
    if (err != qt::noErr)
      throw Error(name(), str::Format(error_movie_add).arg(err));*/
    qt::AddMovieToStorage(_movie, _f);
  }
  _track = 0;
  _media = 0;
  
  //if (_f) { qt::CloseMovieFile(_f); _f = 0; }
  if (_f) { qt::CloseMovieStorage(_f); _f = 0; }
  if (_movie) { qt::DisposeMovie(_movie); _movie = 0; }
  if (_gworld) { qt::DisposeGWorld(_gworld); _gworld = 0; }
}

namespace {

struct _Lock_Pixels
{
  _Lock_Pixels(qt::PixMapHandle in) :
    _p(in)
  {
    qt::LockPixels(_p);
  }

  ~_Lock_Pixels()
  {
    qt::UnlockPixels(_p);
  }

private:

  qt::PixMapHandle _p;
};

}

void Save::set(const Image & in, int) throw (Error)
{
  DJV_DBG_OBJ("Save::set");
  DJV_DBG_PRN("in = " << in);
  
  // Temporary buffers.
  
  gl_data::copy(in, &_image);
  
  // Write.

  const qt::PixMapHandle pixmap =
#if defined(WIN32)
    _gworld->portPixMap;
#else
    qt::GetPortPixMap(_gworld);
#endif
  const qt::Rect rect = { 0, 0, _image.h(), _image.w() };
  long compress_max = 0;

  qt::OSErr err = qt::GetMaxCompressionSize(
    pixmap,
    &rect,
    0,
    quicktime_quality(_option.quality),
    quicktime_codec(_option.codec),
    (qt::CompressorComponent)qt_anyCodec,
    &compress_max);
  if (err != qt::noErr)
    throw Error(name(), str::Format(error_compress).arg(err));

  DJV_DBG_PRN("compress max = " << int(compress_max));

  Handle compress, description;
  compress.init(compress_max);
  description.init(4);

  _Lock_Pixels lock_pixels(pixmap);

  err = qt::CompressImage(
    pixmap,
    &rect,
    quicktime_quality(_option.quality),
    quicktime_codec(_option.codec),
    (qt::ImageDescriptionHandle)description.p,
    *compress.p);
  if (err != qt::noErr)
    throw Error(name(), str::Format(error_compress).arg(err));
  
  err = qt::AddMediaSample(
    _media,
    compress.p,
    0,
    ((qt::ImageDescription *)*description.p)->dataSize,
    _frame_duration,
    (qt::SampleDescriptionHandle)description.p,
    1,
    0,
    0);
  if (err != qt::noErr)
    throw Error(name(), str::Format(error_media_sample).arg(err));
}

const List<Str> & label_save_option()
{
  static const List<Str> data = List<Str>() <<
    "Codec" <<
    "Quality";
  return data;
}

}
}

