/*------------------------------------------------------------------------------
 plugin/djv_tiff/ui.cxx

 Copyright (c) 2004-2008 Darby Johnston. All rights reserved.
 Use is subject to license terms.
------------------------------------------------------------------------------*/

//#define DJV_DBG

#include "ui.h"
#include <djv_ui/choice.h>
#include <djv_ui/group.h>
#include <djv_ui/label.h>
#include <djv_ui/layout_row.h>
#include <djv_ui/style.h>

namespace djv_tiff {
namespace ui {

extern "C" {
DJV_PLUGIN_EXPORT plugin::Plugin * djv_ui_image_save() { return new Save; }
}

struct Save::_Save
{
  choice::Widget * compress;
  choice::Widget * channels;
};

Save::Save() :
  _plugin(0),
  _p(new _Save)
{
  DJV_DBG_OBJ("Save::Save");

  // Widgets.
  
  group::Box * compress_group =
    new group::Box(label_compress_group);
  _p->compress =
    new choice::Widget(djv_tiff::label_compress(), label_compress);
  
  // Layout.
  
  layout::group(List<layout::Item *>() <<
    _p->compress);
  
  Layout_V * layout = new Layout_V(this);

  layout->add(compress_group);
  Layout_V * layout_group = new Layout_V(compress_group);
  layout_group->spacing(style::global()->spacing_large());
  layout_group->add(_p->compress);

  layout->add_stretch();
  
  // Initialize.
  
  plugin_update();
  widget_update();
  
  // Callbacks.
  
  _p->compress->sig.set(this, compress_callback);
}
  
plugin::Plugin * Save::copy() const { return new Save; }

const Str & Save::name() const { return djv_tiff::name; }

void Save::set(djv_image::io::Plugin * in)
{
  _plugin = in;
  
  widget_update();
}

void Save::compress_callback(int in)
{
  if (! _plugin) return;
  
  _option.compress = COMPRESS(in);
  
  plugin_update();
  sig.emit(_plugin);
}

void Save::plugin_update()
{
  callbacks(false);

  if (_plugin)
  {
    Str tmp;
    tmp << _option.compress;
    _plugin->option(_plugin->option_list()[io::SAVE_COMPRESS], &tmp);
  }

  callbacks(true);
}

void Save::widget_update()
{
  callbacks(false);

try
{
  if (_plugin)
  {
    Str tmp;
    tmp = _plugin->option(_plugin->option_list()[io::SAVE_COMPRESS]);
    tmp >> _option.compress;
  }
}
catch (Str) {}

  _p->compress->set(_option.compress);
  
  callbacks(true);
}

const Str
  label_compress_group = "Compress",
  label_compress = "Value:";

}
}

