/*------------------------------------------------------------------------------
 plugin/djv_vlut/save.cxx

 Copyright (c) 2004-2008 Darby Johnston. All rights reserved.
 Use is subject to license terms.
------------------------------------------------------------------------------*/

//#define DJV_DBG

#include "io.h"
#include <FL/x.h>

namespace djv_vlut {
namespace io {

extern "C" {
DJV_PLUGIN_EXPORT plugin::Plugin * djv_image_save() { return new Save; }
}

plugin::Plugin * Save::copy() const { return new Load; }

const Str & Save::name() const { return djv_vlut::name; }

const List<Str> & Save::extension_list() const
  { return djv_vlut::extension_list; }

void Save::open(const File &, const Info &, const Tag &,
  const djv_image::Time &) throw (Error)
{
  DJV_DBG_OBJ("Save::open");
  
  // Open.

#if defined(WIN32)

  _handle.open();

#endif

  // Information.
  
  int size = 0;
  
#if defined(WIN32)

  size = 256;

#elif defined(DJV_IRIX)

  int maps = 0, precision = 0, attr = 0;
  if (! XSGIvcQueryScreenGammaMaps(fl_display, fl_screen, &maps))
    throw Error(name(), djv_image::io::error_open);
  if (maps <= 0)
    throw Error(name(), djv_image::io::error_open);
  if (! XSGIvcQueryGammaMap(fl_display, fl_screen, 0,
    &size, &precision, &attr)
  )
    throw Error(name(), djv_image::io::error_open);
  if (attr & XSGIVC_GMWriteLock)
    throw Error(name(), djv_image::io::error_open);

#else

  if (! XF86VidModeGetGammaRampSize(fl_display, fl_screen, &size))
    throw Error(name(), djv_image::io::error_open);

#endif

  DJV_DBG_PRN("vlut size = " << size);
  
  _info = Info();
  _info.size = Vec2i(size, 1);
  _info.pixel = pixel::RGB_U16;
  DJV_DBG_PRN("info = " << _info);
  
  _image.set(_info);
  _tmp.set(_info);
}

void Save::set(const Image & in, int) throw (Error)
{
  DJV_DBG_OBJ("Save::set");
  DJV_DBG_PRN("in = " << in);

  // Write.
  
  const Data * p = &in;
  if (in.info() != _info)
  {
    DJV_DBG_PRN("convert = " << _image);
    _image.zero();
    gl_data::Option option;
    option.xform.scale = Vec2f(in.size()) / Vec2f(_image.size());
    gl_data::copy(in, &_image, option);
    p = &_image;
  }
  
  data::planar_deinterleave(*p, &_tmp);
  pixel::U16_T * data_p = reinterpret_cast<pixel::U16_T *>(_tmp.data());
  
#if defined(DJV_DBG)
  //for (int i = 0; i < _info.size.x; ++i)
  //{
  //  DJV_DBG_PRN(i << " = " << data_p[i]);
  //{
#endif
  
#if defined(WIN32)

  ::SetDeviceGammaRamp(_handle(), data_p);

#elif defined(DJV_IRIX)

  // XXX
  
  XSGIvcStoreGammaColors16(
    fl_display,
    fl_screen,
    0,
    _info.size.x,
    XSGIVC_MComponentRed | XSGIVC_MComponentGreen | XSGIVC_MComponentBlue,
    data_p);

#else

  if (
    ! XF86VidModeSetGammaRamp(
      fl_display,
      fl_screen,
      _info.size.x,
      data_p,
      data_p + _info.size.x,
      data_p + _info.size.x * 2
    )
  )
    throw Error(name(), djv_image::io::error_write);
  
#endif
}

}
}

