#include <djv_base/user.h>
#include <djv_base/sys.h>
#include <djv_base/file.h>
#include <djv_base/dir.h>
#include <djv_base/app.h>

struct App : public djv_base::app::App
{
  App(int & argc, char ** argv);
};

App::App(int & argc, char ** argv) :
  djv_base::app::App("dir", argc, argv)
{
  using namespace djv_base;
  
  if (argc != 1 && argc != 2)
  {
    print("usage: test_dir [path]");
    exit(app::EXIT_ERROR);
    return;
  }

do
{
  Dir dir(2 == argc ? argv[1] : ".");
  List<File> list = dir.list();
  file::seq_compress(&list);
  file::filter(&list, file::FILTER_HIDDEN);
  file::sort(&list, file::SORT_TIME, true);
  list = file::sort_dir(list);
  
  const int terminal_width = sys::terminal_width();

  for (uint i = 0; i < list.size(); ++i)
  {
    const Str file = list[i].get(-1, false);
    
    const Str info_str = str::Format("%% %% %% %% %%").
      arg(file::label_size(list[i].size)).
      arg(user::uid_to_str(list[i].user)).
      arg(file::label_type()[list[i].type], 4).
      arg(file::label_perm(list[i].perm), 3).
      arg(djv_base::time::time_to_str(list[i].time)
    );

    print(
      str::Format("%% %%").
      arg(file).
      arg(info_str, terminal_width - int(file.size()) - 2)
    );
  }

}
while (0);
}

int main(int argc, char ** argv) { return App(argc, argv).exec(); }

