#include <djv_ui/layout_row.h>
#include <djv_ui/window.h>
#include <djv_ui/button.h>
#include <djv_ui/app.h>

class App : public djv_ui::app::App
{
public:

  App(int & argc, char ** argv);
  ~App();

private:

  DJV_CALLBACK(App, push_callback, bool) { print("push"); }
  DJV_CALLBACK(App, check_callback, bool)
  {
    print(djv_base::str::Format("check = %%").arg(in));
  }
  DJV_CALLBACK(App, radio_callback0, bool) { print("radio 0"); }
  DJV_CALLBACK(App, radio_callback1, bool) { print("radio 1"); }
  DJV_CALLBACK(App, radio_callback2, bool) { print("radio 2"); }
  DJV_CALLBACK(App, tool_callback0, bool) { print("tool"); }
  DJV_CALLBACK(App, tool_callback1, bool)
  {
    print(djv_base::str::Format("tool = %%").arg(in));
  }
  DJV_CALLBACK(App, tool_callback2, bool) { print("tool repeat"); }
  DJV_CALLBACK(App, exit_callback, bool) { exit(); }

  djv_ui::window::Widget * _window;
};

App::App(int & argc, char ** argv) :
  djv_ui::app::App("buttons", argc, argv),
  _window(0)
{
  using namespace djv_base;
  using namespace djv_ui;
  using namespace djv_ui::button;

  valid(true);
  
  // Widgets.
  
  Push * push_button = new Push("&Push Button");
  
  Check * check_button = new Check("&Check Button");
  
  Radio * radio_button0 = new Radio("Radio &1");
  Radio * radio_button1 = new Radio("Radio &2");
  Radio * radio_button2 = new Radio("Radio &3");
  radio_group(List<Radio *>() <<
    radio_button0 <<
    radio_button1 <<
    radio_button2
  );
  
  Tool * tool_button0 = new Tool("file");
  tool_button0->type(Tool::PUSH);
  Tool * tool_button1 = new Tool("dir");
  tool_button1->type(Tool::TOGGLE);
  Tool * tool_button2 = new Tool("magnify");
  tool_button2->type(Tool::REPEAT);

  _window = new djv_ui::window::Widget(name());

  // Layout.
  
  Layout_V * layout = new Layout_V(_window);
  layout->add(push_button);
  layout->add(check_button);
  
  Layout_V * layout_v = new Layout_V(layout);
  layout_v->margin(0);
  layout_v->add(radio_button0);
  layout_v->add(radio_button1);
  layout_v->add(radio_button2);

  Layout_H * layout_h = new Layout_H(layout);
  layout_h->margin(0);
  layout_h->add(tool_button0);
  layout_h->add(tool_button1);
  layout_h->add(tool_button2);
  
  layout->add_stretch();
  
  // Initialize.
  
  radio_button0->set();
  
  _window->size(_window->size_hint());
  _window->show();

  // Callbacks.
  
  push_button->sig.set(this, push_callback);
  check_button->sig.set(this, check_callback);
  radio_button0->sig.set(this, radio_callback0);
  radio_button1->sig.set(this, radio_callback1);
  radio_button2->sig.set(this, radio_callback2);
  tool_button0->sig.set(this, tool_callback0);
  tool_button1->sig.set(this, tool_callback1);
  tool_button2->sig.set(this, tool_callback2);
  _window->close_sig.set(this, exit_callback);
}

App::~App()
{
  if (_window) _window->del();
}

int main(int argc, char ** argv) { return App(argc, argv).exec(); }

